local type = type
local error = error
local unpack = unpack
local strformat = string.format

local lfuncs = {}
local cfuncs = {}
local function create (name, ...)
	local f = lfuncs[name]
	if type(f) == 'function' then
		return f(...)
	end
	local f = cfuncs[name]
	if type(f) == 'table' then
		return f:create(...)
	end
	error(strformat('unknown action type `%s`', name), 2)
end

lfuncs.Sequence = function (...)
	local actions = CCArray:create()
	for _, cfg in ipairs({...}) do
		actions:addObject(create(unpack(cfg)))
	end
	return CPSequence:create(actions)
end
lfuncs.Spawn = function (...)
	local actions = CCArray:create()
	for _, cfg in ipairs({...}) do
		actions:addObject(create(unpack(cfg)))
	end
	return CCSpawn:create(actions)
end
lfuncs.RepeatForever = function (cfg)
	return CPRepeatForever:create(create(unpack(cfg)))
end
lfuncs.MoveTo = function (d, x, y)
	return CPMoveTo:create(d, ccp(x, y))
end
lfuncs.MoveBy = function (d, x, y)
	return CPMoveBy:create(d, ccp(x, y))
end
lfuncs.JumpTo = function (d, x, y, h, n)
	return CPJumpTo:create(d, ccp(x, y), h, n)
end
lfuncs.Speed = function (cfg, s)
	return CPSpeed:create(create(unpack(cfg)), s)
end
lfuncs.TintAddTo = function (d, r, g, b)
	return CPTintTo:createForAddColor(d, r, g, b)
end
lfuncs.Animate = function (name, plist, dpu, frame)
	local r = CPAnimate:create(CreateAnimationOnly(name, plist, dpu, frame), '', nil)
	return r
end
lfuncs.AnimateEx = function (name, plist, sfx)
	return CPAnimate:createAnimateByName(name, plist, sfx)
end
lfuncs.BezierBy = function (d, cp1, cp2, ep)
	local bc = ccBezierConfig()
	bc.controlPoint_1 = ccp(cp1[1], cp1[2])
	bc.controlPoint_2 = ccp(cp2[1], cp1[2])
	bc.endPosition = ccp(ep[1], ep[2])
	return CCBezierBy:create(d, bc)
end
lfuncs.EaseExponentialIn = function (...)
	return CCEaseExponentialIn:create(create(...))
end
lfuncs.EaseExponentialOut = function (...)
	return CCEaseExponentialOut:create(create(...))
end

cfuncs.FadeOut = CPFadeOut
cfuncs.FadeIn = CPFadeIn
cfuncs.ScaleTo = CPScaleTo
cfuncs.ScaleBy = CCScaleBy
cfuncs.RotateTo = CPRotateTo
cfuncs.RotateBy = CPRotateBy
cfuncs.TintTo = CPTintTo
cfuncs.ClipTo = CPClipTo
cfuncs.DelayTime = CPDelayTime

cfuncs.Show = CPShow
cfuncs.Hide = CPHide
cfuncs.CallFunc = CPCallFunc
cfuncs.CallFuncN = CPCallFuncN
cfuncs.RemoveSelf = CPRemoveSelf

return {create = create}