local proto = require'protocol'
local itemCfg = require'config.item'
local tcast = tolua.cast
local SSFmt = string.format

LayerMall = class('LayerMall', CppUI)
local super = LayerMall.__super
local m_LayerMallrefreshCost = 50
local m_LayerMallmaxRefreshCost = 1000
local m_LayerMallisInActivity = false
local m_LayerMalltotalfreetimes = 0
local m_LayerMallrefreshtime = 0
local g_MoneyCost = 0
local g_CristalCost = 0

local function ConfirmRefreshItemList()
	local msg = proto.C2S_MallRefreshItemList.new()
	g_SendMsg(msg)
end

function LayerMall:init()
	if not super.init(self) then return false end
	
	return true
end

function LayerMall:onEnter()
	local btn_refresh = self:seekWidgetByName('btn_refresh')
	if (btn_refresh ~= nil) then 
		self:addLuaHandler('btn_refresh', function () self:OnRefresh() end)
	end
	
	self:regMsgHandler('mallsync', self.CostInfo)
	self:regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_MallSyncItemList, self.OnRecvMsgMallSyncItemList)
	return true
end

function LayerMall:onEnterDidFinish()
	self:addLuaHandler('btn_buyall', function () 
		local tip
		if g_MoneyCost ~= 0 then
			if g_CristalCost ~= 0 then
				tip = SSFmt(TDGETSTRCS('MALL_CONFIRM_BUY_ALL_3'), g_MoneyCost, g_CristalCost)
			else
				tip = SSFmt(TDGETSTRCS('MALL_CONFIRM_BUY_ALL_1'), g_MoneyCost)
			end
		else
			if g_CristalCost ~= 0 then
				tip = SSFmt(TDGETSTRCS('MALL_CONFIRM_BUY_ALL_2'), g_CristalCost)
			else
				tip = SSFmt(TDGETSTRCS('MALL_CONFIRM_BUY_ALL'))
			end
		end
		
		LShowMessage(tip,
			nil, nil, MBT_OKCANCEL,
			function ()
				local msg = proto.C2S_MallBuyItemAll.new()
				g_SendMsg(msg)
			end) 
	end)
end

local colors = {
	{ 255, 255, 255 },
	{ 140, 255, 130 },
	{ 100, 150, 255 },
	{ 255, 0, 255 },
	{ 255, 127, 0 },
	{ 255, 0, 0 },
}
function LayerMall:OnRecvMsgMallSyncItemList( msg ) 
	g_MoneyCost = 0
	g_CristalCost = 0
	
	performWithDelay(self, function ()
		local nCount = #msg.itemList
		if nCount > 6 then
			nCount = 6
		end
		
		local scroll = self:seekWidgetByName("list_item")
		for k = 1, nCount do
			local widget = tcast(scroll:getChildByTag(k-1), 'Widget')
			local label = UIHO:seekWidgetByName(widget, "lbl_name")
			local item = msg.itemList[k+1]
			local cfg = itemCfg[item.itemTemplateId]
			local quity = 0
			if label ~= nil and cfg ~= nil then
				if cfg.itemClass == ITEM_CLASS_EQUIPMENT then
					quity = item.level % 100 - 1
				end
				if quity > 5 then
					quity = 5
				end
				label:setColor(ccc3(colors[quity+1][1],colors[quity+1][2],colors[quity+1][3]))
			end
			self:AllItemCost(item.useCoin, item.discount, item.price)
		end
		
	end, 0.001)
end

function LayerMall:AllItemCost(useCoin, discount, cost)
	local price = cost
	if discount == 1 then
		price = price / 10
	elseif discount == 2 then
		price = price / 2
	elseif discount == 3 then
		price = price * 8 / 10
	end
	
	if useCoin == 1 then
		g_MoneyCost = g_MoneyCost + price
	else
		g_CristalCost = g_CristalCost + price
	end
end

function LayerMall:CostInfo(msg)
	m_LayerMallrefreshCost = msg.count
	m_LayerMallmaxRefreshCost = msg.price
	m_LayerMalltotalfreetimes = msg.freetimes
	m_LayerMallrefreshtime = msg.refreshtimes
	if (msg.activity == 1) then
		m_LayerMallisInActivity = true
	else
		m_LayerMallisInActivity = false
	end
	
	local img = self:seekWidgetByName("Image_29")
	local label = UIHO:seekWidgetByName(img, "lab_money")
	label:setText(SSFmt("%u", PLAYER_BASE_DATA.money))
	
	return true
end

function LayerMall:GetRefreshCost(refreshtime)
	local cost = 0
	
	if m_LayerMallisInActivity then
		cost = m_LayerMallrefreshCost
	else
		if (m_LayerMalltotalfreetimes > 0) and (refreshtime < m_LayerMalltotalfreetimes) then
			cost = 0 
		else
			cost = (refreshtime + 1 - m_LayerMalltotalfreetimes) * m_LayerMallrefreshCost
		end
	end
	
	if (cost > m_LayerMallmaxRefreshCost)then
		cost = m_LayerMallmaxRefreshCost
	end
	
	return cost;
end

function LayerMall:OnRefresh()
	local cost = self:GetRefreshCost(m_LayerMallrefreshtime);
	--免费
	if (cost == 0) then
		LShowMessage(SSFmt(TDGETSTRCS("MALL_CONFIRM_REFRESH_VIP"), m_LayerMallrefreshtime + 1), 
					nil, nil, MBT_OKCANCEL, ConfirmRefreshItemList)
	--收费
	else
		LShowMessage(SSFmt(TDGETSTRCS("MALL_CONFIRM_REFRESH"), cost, m_LayerMallrefreshtime + 1),
					nil, nil, MBT_OKCANCEL, ConfirmRefreshItemList)
	end
	
	return true
end