local tcast = tolua.cast
local proto = require'protocol'
local itemCfg = require'config.item'

local cVer = DlgMailBox
DlgMailBox = class('DlgMailBox', CppUI)
local super = DlgMailBox.__super

-- local g_mailList = {}
local colorStr = {'888888', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}

local function getColorName(id, color)
	local cfg = itemCfg[id]
	if not cfg then return 'null' end
	
	local star = math.floor(color / 100)
	local color = color % 100 + 1
	if color > 8 then color = 8 end
	
	local nameEx = ''
	if star > 0 then
		nameEx = nameEx..SSFmt(TDGETSTRCS('EQUIP_STAR_NAME'), star)
	end
	if color >= 6 and color <= 8 then
		nameEx = nameEx..TDGETSTRCS('EQUIP_SQ_NAME_'..color)
	end
	if nameEx ~= '' then nameEx = '['..nameEx..']' end
	
	if color > 6 then color = 6 end
	return SSFmt('<c=%s>%s%s</>', colorStr[color], cfg.name, nameEx)
end

function DlgMailBox:onEnter()
	self:regJMsgHandler('jsonmail', self.onRecvMailList)
end

function DlgMailBox:onRecvMailList(msg)
	performWithDelay(self, function ()
		local lv = self:seekWidgetByName('maillist')
		for k, v in ipairs(msg) do
			local item = tcast(lv:getChildByTag(v.id), 'Widget')
			local panel = UIHO:seekWidgetByName(item, 'txt')
			if panel:getChildByTag(9527) ~= nil then panel:removeChildByTag(9527, true) end
			local label = UIStaticRichText:createWithString('', panel:getSize().width, 3)
			label:setAnchorPoint(ccp(0, 0))
			panel:addChild(label, 999, 9527)
			local txt = v.text..'\n'
			if v.attachment ~= 0 or
				v.money ~= 0 or
				v.crystal ~= 0 or
				v.honor ~= 0 or
				v.credit ~= 0 then
				txt = txt..TDGETSTRCS("SD02_MAIL_AWARD")
			end
			if v.money ~= 0 then
				txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_MONEY"), v.money)
			end
			if v.crystal ~= 0 then
				txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_CRYSTAL"), v.crystal)
			end
			if v.honor ~= 0 then
				txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_HONOR"), v.honor)
			end
			if v.credit ~= 0 then
				txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_CRIDET"), v.credit)
			end
			if v.attachment ~= 0 then
				local templateId = v.attachment % 65536
				local count = math.floor(v.attachment / 65536)
				local cfg = itemCfg[templateId]
				if cfg then
					if cfg.itemClass == ITEM_CLASS_EQUIPMENT then
						txt = txt..getColorName(templateId, count)
					elseif cfg.itemClass == ITEM_CLASS_SUPPLEMENT then
						txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_RUNEITEM"),g_Player:GetRuneItemLevel(count),cfg.name)
					else
						txt = txt..SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_ITEM"), cfg.name, count)
					end
				end
			end
			label:setText(txt)
		end
	end, 0.0001)
end

function DlgMailBox:Show(...)
	cVer:Show(...)
end
