require'cppui.cppui'
require'cppui.payactivity'
require'cppui.factionlist'
require'cppui.factionmember'
require'cppui.factionbattle'
require'cppui.battlemap'
require'cppui.remakeequipment'
require'cppui.tobstronger'
require'cppui.viewplayer'
require'cppui.equipremake'
require'cppui.equipselect'
require'cppui.mailbox'
require'cppui.charge'
require'cppui.gemlevelup'
require'cppui.battelstatistics'
require'cppui.layermall'
require'cppui.layerroot'
require'cppui.layerbattle'
require'cppui.layerbag'
require'cppui.layerequip'
require'cppui.layerfaction'
require'cppui.layerpet'
require'cppui.layerelitefight'
require'cppui.vipdetail'
require'cppui.playerinfo'
require'cppui.itemdetail'
require'cppui.layergroupfight'
require'cppui.dlgbattlelist'

local runeCfg = require'config.rune'
local proto = require'protocol'

local sqAbility = {200,300,10,150,200,500,500,1000,300,500}
local sq2Ability = {250,200,200,200,200,250,200,200,250,200}

local sqName = {
	'MAIN_WEAPON',
	'SUB_WEAPON',
	'HEAD',
	'BODY',
	'LEG',
	'GLOVE',
	'RING',
	'NECKLACE',
	'WAIST',
	'BOOTS',
}

function buildSqAttrDesc(i, lv)
	local name = getString('ST_SPECIAL_'..sqName[i+1])
	return string.format(name, sqAbility[i+1]*lv / 100)..'\n'
end
function buildSqAttrDescWithVal(i, val)
	local name = getString('ST_SPECIAL_'..sqName[i+1])
	return string.format(name, val / 100)..'\n'
end
function buildSq2AttrDesc(i, lv)
	local name = getString('SYNC_PROP_SQ2_'..i)
	return string.format('%s+%.02f%%\n', name, sq2Ability[i+1]*lv / 100)
end
function buildSq2AttrDescWithVal(i, val)
	local name = getString('SYNC_PROP_SQ2_'..i)
	return string.format('%s+%.02f%%\n', name, val / 100)
end
function buildStarAttrDesc(i, val)
	local name = getString('SD_STAR_ATTR_'..i)
	return string.format('%s+%.02f%%\n', name, val / 100)
end
function buildRuneAttrDesc(i, val)
	local name = getString('RUNE_SPECIAL_'..i)
	return string.format('%s+%.02f%%\n', name, val / 100)
end

function GetAttrDetail()
	local pData = DPlayerData:GetInstance()
	local enhancedLv = pData:getFullEnhancedLevel()
	
	local sqAbi, sq2Abi, starAbi, runeAbi = {}, {}, {}, {}
	for k = EQUIPMENT_TYPE_MAIN_WEAPON, EQUIPMENT_TYPE_BOOTS do
		local equip = pData:GetEquipmentData(k)
		if equip ~= nil then
			-- 第一神器属性
			local speLv = equip:GetSpecialLevel(equip.m_specialProperty)
			if speLv >= 0 then
				local enhLv = math.min(speLv, enhancedLv)
				sqAbi[k] = sqAbility[k + 1] * (enhLv + speLv)
			end
			-- 第二神器属性
			local spe2Lv = equip:Get2SpecialLevel(equip.m_2specialProperty)
			if spe2Lv >= 0 then
				sq2Abi[k] = sq2Ability[k + 1] * spe2Lv
			end
			-- 星星属性
			if equip:GetStarNum() > 0 then
				local idx = equip:GetStarAttrType()
				local val = equip:GetStarAttrValue()
				starAbi[idx] = (starAbi[idx] or 0) + val
			end
		end
	end
	
	--下面把服务附近的属性算上
	for k=1, #g_Player.runeinventory_pos do
	  local runeitem, where = g_Player:GetRuneItem(g_Player.runeinventory_pos[k])
	  if runeitem and where == proto.PACKAGE_TYPE_RUNEINVENTORY then
		local runelv = runeitem:Level()
		local _runeCfg = runeCfg[runeitem.templateId]
		if _runeCfg and _runeCfg.abitype == 1 then
			if _runeCfg.valueindex >=  proto.ABILITY_TYPE_S_HPRECOVER then
				runeAbi[_runeCfg.valueindex] = _runeCfg[string.format('value%d',runelv)]
			end
		elseif _runeCfg and _runeCfg.abitype == 2 then
			sq2Abi[_runeCfg.valueindex] = sq2Abi[_runeCfg.valueindex] + _runeCfg[string.format('value%d',runelv)]
		end
	  end
	end
	
	local isFirst, str1 = true, ''
	for k = EQUIPMENT_TYPE_MAIN_WEAPON, EQUIPMENT_TYPE_BOOTS do
		if sqAbi[k] and sqAbi[k] > 0 then
			if isFirst then
				str1 = str1..getString('SD_FULL_SQTETAILS')
				isFirst = false
			end
			str1 = str1..buildSqAttrDescWithVal(k, sqAbi[k])
		end
		if sq2Abi[k] and sq2Abi[k] > 0 then
			if isFirst then
				str1 = str1..getString('SD_FULL_SQTETAILS')
				isFirst = false
			end
			str1 = str1..buildSq2AttrDescWithVal(k, sq2Abi[k])
		end
	end
	str1 = string.sub(str1, 1, #str1-1)
	
	local isFirst, str2 = true, ''
	for k = 0, 1 do
		if starAbi[k] and starAbi[k] > 0 then
			if isFirst then
				str2 = str2..getString('SD_FULL_STARDETAILS')
				isFirst = false
			end
			str2 = str2..buildStarAttrDesc(k, starAbi[k])
		end
	end
	str2 = string.sub(str2, 1, #str2-1)
	
	local isFirst, str3 = true, ''
	for k = proto.ABILITY_TYPE_S_HPRECOVER, proto.ABILITY_TYPE_S_DAMAGE_APP do
		if runeAbi[k] and runeAbi[k] > 0 then
			if isFirst then
				str3 = str3..getString('SD_FULL_RUNETAILS')
				isFirst = false
			end
			str3 = str3..buildRuneAttrDesc(k, runeAbi[k])
		end
	end
	str3 = string.sub(str3, 1, #str3-1)
	
	return str1..str2..str3
end