local MAIN_TYPE_FightBefore = 0
local MAIN_TYPE_Fighting = 1
local MAIN_TYPE_FightAfter = 2

local SUB_TYPE_ResChange = 0
local SUB_TYPE_BuffChange = 1
local SUB_TYPE_Dizzy = 2
local SUB_TYPE_Relive = 3

FIGHT_TYPE_Normal = 0
FIGHT_TYPE_BattleField = 1
FIGHT_TYPE_EliteMap = 2
FIGHT_TYPE_Quarry = 3
FIGHT_TYPE_Abyss = 4

local ActionTag = -100
local tcast = tolua.cast

local bit = require'bit'
local proto = require'protocol'
local itemCfg = require'config.item'
local skillCfg = require'config.skill'
local actionMgr = require'action'


local FRAME_RESULT_TYPE_FRT_RELIVE = proto.FRAME_RESULT_TYPE_FRT_POISON + 1 --

local HPMP_RESULT_TYPE_EX_HRT_DECMP_EX = proto.HPMP_RESULT_TYPE_HRT_DECMP +1 --

local ATTACK_RESULT_TYPE_EX_ART_DRB_EX = proto.ATTACK_RESULT_TYPE_ART_DECMP_CRIT + 1 --񵲷

local ATTACK_RESULT_TYPE_EX_ART_MISS_EX = ATTACK_RESULT_TYPE_EX_ART_DRB_EX +1 --

local ATTACK_RESULT_TYPE_EX_ART_CRIT_EX = ATTACK_RESULT_TYPE_EX_ART_MISS_EX + 1--Ա

FightObjData = class('FightObjData')

function FightObjData:ctor()
	self.panel = nil
	self.curHp = 0
	self.curMp = 0
	self.curRound = 0
	self.colorName = ''
	self.buffList = {}
	self.isLeft = false
end

function FightObjData:GetBuffStr()
	local visiable = false
	local buffStr = '['
	for k, v in pairs(self.buffList) do
		local cfg = skillCfg.GetBuffCfg(v.fbt)
		if cfg then
			visiable = true
			buffStr = buffStr..SSFmt('%s%u', cfg.name, v.round_cd)
			buffStr = buffStr..' '
		end
	end
	buffStr = string.sub(buffStr, 1, -2)..']'
	return visiable, buffStr
end

function FightObjData:RunBuff()
	for k = #self.buffList, 1, -1  do
		local v = self.buffList[k]
		if v.round_cd > 0 then
			v.round_cd = v.round_cd - 1
		end
		if v.round_cd == 0 then
			table.remove(self.buffList, k)
		end
	end
end

function FightObjData:AddBuff(buff)
	table.insert(self.buffList, buff)
end

function FightObjData:DecMp(num)
	self.curMp = math.max(0, self.curMp - num)
end
function FightObjData:DecHp(num)
	self.curHp = math.max(0, self.curHp - num)
end
function FightObjData:AddHp(num)
	self.curHp = self.curHp + num
end
function FightObjData:AddMp(num)
	self.curMp = self.curMp + num
end
function FightObjData:FloatText(fontType, prefix, value, delay,txt)
	local panel = self.panel
	if not panel then return end
	if not delay then delay = 0.2 end
	
	local size = panel:getSize()
	local pLv
	local info = SSFmt('%s%u', prefix, value)
	if fontType == 'miss' then
		local miss = UIStaticRichText:createWithString(txt)
		miss:setScale(2)
		miss:setVisible(false)
		local cp1 = {-100, 20}
		local cp2 = {100, 40}
		local ep = {-100, 50}
		if not self.isLeft then
			cp1[1] = 100
			cp2[1] = -100
			ep[1] = 100
		end
		miss:runAction(actionMgr.create(
			'Sequence',
			{'DelayTime', delay},
			{'Show'},
			{'BezierBy', 1, cp1, cp2, ep},
			{'CallFunc', function () panel:getParent():removeChild(miss) end}
		))
		miss:setPosition(panel:getParent():convertToNodeSpace(panel:convertToWorldSpace(ccp(size.width/2, size.height/2))))
		panel:getParent():addChild(miss, 100, ActionTag)
		return
	elseif fontType == 'num_goods2.png' then
		pLv = CCLabelAtlas:create(SSFmt('%u', value), fontType, 17, 32, 0x30)
	elseif fontType == 'num_renew1.png' then
		pLv = CCLabelAtlas:create(SSFmt('%u', value), fontType, 30, 34, 0x30)
	else
		pLv = CCLabelBMFont:create(info, fontType)
	end
	pLv:setAnchorPoint(ccp(0.5, 0.5))
	pLv:setPosition(size.width / 2, size.height / 2)
	local isAdd = false
	if prefix ~= nil and string.byte(prefix, 1) == 0x2b then
		isAdd = true
		pLv:setPositionY(pLv:getPositionY() - 50)
	end
	pLv:setVisible(false)
	local removeSelf = function ()
		tcast(pLv:getParent(), 'Widget'):removeNode(pLv)
	end
	if not isAdd then
		pLv:setScale(2)
		pLv:runAction(actionMgr.create(
			'Sequence',
			{'DelayTime', delay},
			{'Show'},
			{'MoveBy', 0.3, 0, 50},
			{'DelayTime', 1},
			{'FadeOut', 0.5},
			{'CallFunc', removeSelf}
		))
	else
		pLv:setScale(1)
		pLv:runAction(actionMgr.create(
			'Sequence',
			{'DelayTime', delay},
			{'Show'},
			{'MoveBy', 0.3, 0, 50},
			{'DelayTime', 0.2},
			{'CallFunc', removeSelf}
		))
		pLv:runAction(actionMgr.create(
			'Sequence',
			{'DelayTime', delay},
			{'ScaleTo', 0.3, 2},
			{'DelayTime', 0.1},
			{'ScaleTo', 0.1, 1}
		))
	end
	pLv:setPosition(panel:getParent():convertToNodeSpace(panel:convertToWorldSpace(ccp(pLv:getPosition()))))
	tcast(panel:getParent(), 'Widget'):addNode(pLv, 100, ActionTag)
end

FightMgr = class('FightMgr')

function FightMgr:ctor()
	self.m_curFightMsg = nil
	self.curFrame = 0
	self.ifdList = {}
	self.fodMap = {}
	self.battleInfo = {}
	self.waitTicks = 0
	self.fightType = -1
end

function FightMgr:GetFod(id)
	return self.fodMap[id]
end

function FightMgr:GetBattleInfo()
	local infoString = ''
	for k, v in ipairs(self.battleInfo) do
		infoString = infoString..v..'\n'
	end
	return infoString
end

function FightMgr:Reset()
	self.m_curFightMsg = nil
end

function FightMgr:Valid()
	return self.m_curFightMsg ~= nil
end

function FightMgr:SetFightType(ft)
	self.fightType = ft
end

function FightMgr:GetFightMsg()
	return self.m_curFightMsg
end

function FightMgr:GetPrevFightMsg()
	return self.msg
end

function FightMgr:getCurFrame()
	return self.curFrame
end

function FightMgr:GetFightResult()
	if self:Valid() then
		return self.m_curFightMsg.fightResult
	else
		return nil
	end
end

function FightMgr:SetMagData(msg)
	self:Reset()
	self.msg = msg
	self.m_curFightMsg = msg
	self.curFrame = 0
	self.waitTicks = 0
end

function FightMgr:FrameEmpty()
	if not self:Valid() then
		return true
	end
	return table.empty(self.m_curFightMsg.frameList)
end

function FightMgr:ShowMsg()
	if self:Valid() then
		if self.curFrame >= #self.m_curFightMsg.frameList and
			table.empty(self.ifdList) then
			self:UpdateFightInnerFrame(nil)
			self:Reset()
			return
		end
		if table.empty(self.ifdList) then
			self.curFrame = self.curFrame + 1
			local frameData = self.m_curFightMsg.frameList[self.curFrame]
			self:UpdateFightFrame(frameData)
		end
		if table.empty(self.ifdList) then
			return
		end
		local ifd = table.remove(self.ifdList, 1)
		self:UpdateFightInnerFrame(ifd)
		if ifd.mainType == MAIN_TYPE_Fighting or
			ifd.mainType == MAIN_TYPE_FightAfter then
			while not table.empty(self.ifdList) do
				ifd = table.remove(self.ifdList, 1)
				self:UpdateFightInnerFrame(ifd)
			end
		end
	elseif self.curFrame > 0 then
		self:UpdateFightInnerFrame(nil)
	end
end

function FightMgr:FightBegin()
	local fodMap = {}
	self.fodMap = fodMap
	local battleInfo = self.battleInfo
	
	local msg = self:GetFightMsg()
	if msg == nil then return end
	
	if #msg.objsLeft == 0 then return end
	
	battleInfo[#battleInfo+1] = TDGETSTRCS('ST_BATTLE_BEGIN')
	
	local function InitFightObjectDetails(__fod, __color, __isLeft)
		local _fod = __fod
		local fod = fodMap[_fod.id]
		if not fod then
			fod = FightObjData.new()
			fodMap[_fod.id] = fod
		end
		for k, v in pairs(_fod) do
			fod[k] = v
		end
		fod.curHp = _fod.hp
		fod.curMp = _fod.mp
		fod.panel = nil
		fod.curRound = 0
		fod.colorName = SSFmt('<c=%s>%s</>', __color, fod.name)
		fod.isLeft = __isLeft
	end
	local fightType = self.fightType
	if fightType == FIGHT_TYPE_Normal then
		InitFightObjectDetails(msg.objsLeft[1], 'ff8800', true)
		if #msg.objsLeft == 2 then
			InitFightObjectDetails(msg.objsLeft[2], '888800', true)
		elseif #msg.objsLeft == 3 then
			InitFightObjectDetails(msg.objsLeft[2], '888800', true)
			InitFightObjectDetails(msg.objsLeft[3], '888800', true)
		end
		for k, v in ipairs(msg.objsRight) do
			if k > 3 then break end
			InitFightObjectDetails(v, '8888ff', false)
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('ST_FIND_TARGET'), v.name, v.levle, v.hp)
		end
	elseif fightType == FIGHT_TYPE_BattleField then
		for k, v in ipairs(msg.objsLeft) do
			if k > 3 then break end
			InitFightObjectDetails(v, 'ff8800', true)
		end
		for k, v in ipairs(msg.objsRight) do
			if k > 3 then break end
			InitFightObjectDetails(v, '8888ff', false)
		end
	elseif fightType == FIGHT_TYPE_EliteMap then
		for k, v in ipairs(msg.objsLeft) do
			if k > 9 then break end
			InitFightObjectDetails(v, 'ff8800', true)
		end
		for k, v in ipairs(msg.objsRight) do
			if k > 1 then break end
			InitFightObjectDetails(v, '8888ff', false)
		end
	elseif fightType == FIGHT_TYPE_Quarry then
		for k, v in ipairs(msg.objsLeft) do
			if k > 3 then break end
			InitFightObjectDetails(v, 'ff8800', true)
		end
		for k, v in ipairs(msg.objsRight) do
			if k > 3 then break end
			InitFightObjectDetails(v, '8888ff', false)
		end
	elseif fightType == FIGHT_TYPE_Abyss then
		for k, v in ipairs(msg.objsLeft) do
			if k > 3 then break end
			InitFightObjectDetails(v, 'ff8800', true)
		end
		for k, v in ipairs(msg.objsRight) do
			if k > 2 then break end
			InitFightObjectDetails(v, '8888ff', false)
		end
	end
	
	if self.panel then
		self.panel:FightBegin(self)
	end
end

function FightMgr:UpdateFightFrame(ffd)
	local tempifdList = {}
	local fodMap = self.fodMap
	
	local roundtip = SSFmt(TDGETSTRCS('FIGHT_ROUNT'), self.m_curFightMsg.tround - ffd.round)
	local attacker = fodMap[ffd.attid]
	
	for k, v in ipairs(ffd.hpmpBeforn) do
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		
		local attData = v
		ifd.who = ffd.attid
		ifd.target = attData.id
		ifd.mainType = MAIN_TYPE_FightBefore
		ifd.subType = SUB_TYPE_ResChange
		ifd.resType = attData.result
		ifd.value = attData.att_value
		ifd.info = ''
		
		if ffd.attid == ifd.target and ffd.id_value ~= 0 and
			attData.result == proto.HPMP_RESULT_TYPE_HRT_DECMP then
			if ffd.id_value ~= 0 then
				ifd.param = ffd.id_value
				ifd.paramPos = {}
				for k, v in ipairs(ffd.objList) do
					local attData = v
					local defender = fodMap[attData.id]
					local iv = UIHO:seekWidgetByName(defender.panel, 'img_icon')
					if iv then
						local pos = iv:convertToWorldSpace(ccp(0, 0))
						ifd.paramPos[k] = attacker.panel:getParent():convertToNodeSpace(pos)
					end
				end
			end
		end
		tempifdList[#tempifdList+1] = ifd
	end
	
	local frameResult = ffd.frameResult
	if frameResult == proto.FRAME_RESULT_TYPE_FRT_ATTACK then
		if ffd.special_op == 0 then
			for k, v in ipairs(ffd.objList) do
				local ifd = {}
				ifd.param = 0
				ifd.roundtip = roundtip
				
				local attData = v
				local defender = fodMap[attData.id]
				local skillName = TDGETSTRCS('BAT_NORMALSKILL_NAME')
				local cfg = skillCfg[ffd.id_value]
				if cfg then skillName = cfg.name end
				if attData.result == proto.ATTACK_RESULT_TYPE_ART_HIT then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECHP
					ifd.info = SSFmt(TDGETSTRCS('BAT_NORMAL_ATT'), attacker.colorName, skillName, defender.colorName, attData.att_value)
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_MISS then
					ifd.resType = -1
					ifd.info = SSFmt(TDGETSTRCS('BAT_NORMAL_MISS'), attacker.colorName, skillName, defender.colorName)
				elseif attData.result == ATTACK_RESULT_TYPE_EX_ART_MISS_EX then --
					ifd.resType = -2
					ifd.info = SSFmt(TDGETSTRCS('BAT_NORMAL_MISS'), attacker.colorName, skillName, defender.colorName)
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_CRIT then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECHP
					ifd.info = SSFmt(TDGETSTRCS('BAT_CRIT_ATT'), attacker.colorName, skillName, defender.colorName, attData.att_value)
				elseif attData.result == ATTACK_RESULT_TYPE_EX_ART_CRIT_EX then--Ա
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECHP
					ifd.info = SSFmt(TDGETSTRCS('BAT_CRIT_ATT'), attacker.colorName, skillName, defender.colorName, attData.att_value)
					ifd.rune_crit = true
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_ADDHP then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_ADDHP
					if ffd.attid == attData.id then
						ifd.info = SSFmt(TDGETSTRCS('BAT_SELF_ADDHP'), attacker.colorName, skillName, attData.att_value)
					else
						ifd.info = SSFmt(TDGETSTRCS('BAT_OTHER_ADDHP'), attacker.colorName, skillName, defender.colorName, attData.att_value)
					end
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_ADDMP then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_ADDMP
					if ffd.attid == attData.id then
						ifd.info = SSFmt(TDGETSTRCS('BAT_SELF_ADDMP'), attacker.colorName, skillName, attData.att_value)
					else
						ifd.info = SSFmt(TDGETSTRCS('BAT_OTHER_ADDMP'), attacker.colorName, skillName, defender.colorName, attData.att_value)
					end
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_DECMP then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECMP
					ifd.info = SSFmt(TDGETSTRCS('BAT_NORMAL_DECMP'), attacker.colorName, skillName, defender.colorName, attData.att_value)
				elseif attData.result == proto.ATTACK_RESULT_TYPE_ART_DECMP_CRIT then
					ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECMP
					ifd.info = SSFmt(TDGETSTRCS('BAT_NORMAL_DECMP_CRIT'), attacker.colorName, skillName, defender.colorName, attData.att_value)
				--񵲷
				elseif attData.result == ATTACK_RESULT_TYPE_EX_ART_DRB_EX then
					ifd.resType = ATTACK_RESULT_TYPE_EX_ART_DRB_EX
					ifd.info = SSFmt(TDGETSTRCS('BAT_RUNESKILL_DRB'), defender.colorName)
				end
				ifd.who = ffd.attid
				ifd.target = attData.id
				ifd.mainType = MAIN_TYPE_Fighting
				ifd.subType = SUB_TYPE_ResChange
				ifd.value = attData.att_value
				ifd.param = attData.result
				tempifdList[#tempifdList+1] = ifd
			end
		end
	elseif frameResult == proto.FRAME_RESULT_TYPE_FRT_SKIP then
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		if ffd.id_value == proto.FIGHT_BUFF_TYPE_FBT_VERTIGO then
			ifd.info = SSFmt(TDGETSTRCS('BAT_SKILL_VERTIGO'), attacker.colorName)
		elseif ffd.id_value == proto.FIGHT_BUFF_TYPE_FBT_FROZEN then
			ifd.info = SSFmt(TDGETSTRCS('BAT_SKILL_FROZEN'), attacker.colorName)
		elseif ffd.id_value == proto.FIGHT_BUFF_TYPE_FBT_HIDE then
			ifd.info = SSFmt(TDGETSTRCS('BAT_SKILL_HIDE'), attacker.colorName)
		end
		ifd.who = ffd.attid
		ifd.target = ffd.attid
		ifd.mainType = MAIN_TYPE_Fighting
		ifd.subType = SUB_TYPE_Dizzy
		ifd.resType = 0
		ifd.value = ffd.id_value
		tempifdList[#tempifdList+1] = ifd
	elseif frameResult == proto.FRAME_RESULT_TYPE_FRT_FIRESTORM then
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		ifd.info = SSFmt(TDGETSTRCS('BAT_SKILL_FIRESTORM'), attacker.colorName, ffd.id_value)
		ifd.who = ffd.attid
		ifd.target = ffd.attid
		ifd.mainType = MAIN_TYPE_Fighting
		ifd.subType = SUB_TYPE_ResChange
		ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECHP
		ifd.value = ffd.id_value
		tempifdList[#tempifdList+1] = ifd
	elseif frameResult == proto.FRAME_RESULT_TYPE_FRT_POISON then
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		ifd.info = SSFmt(TDGETSTRCS('BAT_SKILL_POISON'), attacker.colorName, ffd.id_value)
		ifd.who = ffd.attid
		ifd.target = ffd.attid
		ifd.mainType = MAIN_TYPE_Fighting
		ifd.subType = SUB_TYPE_ResChange
		ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECHP
		ifd.value = ffd.id_value
		tempifdList[#tempifdList+1] = ifd
	elseif frameResult == FRAME_RESULT_TYPE_FRT_RELIVE then --Ǹ
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		ifd.info = SSFmt(TDGETSTRCS('BAT_RUNESKILL_RELIVE'), attacker.colorName)
		ifd.who = ffd.attid
		ifd.target = ffd.attid
		ifd.mainType = MAIN_TYPE_Fighting
		ifd.subType = SUB_TYPE_Relive
		ifd.resType = 0
		ifd.value = 0
		tempifdList[#tempifdList+1] = ifd
	end
	
	for k, v in ipairs(ffd.buffList) do
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		local buffData = v
		ifd.who = ffd.attid
		ifd.target = buffData.id
		ifd.mainType = MAIN_TYPE_FightAfter
		ifd.subType = SUB_TYPE_BuffChange
		ifd.resType = buffData.fbt
		ifd.value = buffData.round_cd
		ifd.info = ''
		tempifdList[#tempifdList+1] = ifd
	end
	
	local addHpdelay = 0
	for k, v in ipairs(ffd.hpmpAafter) do
		local ifd = {}
		ifd.param = 0
		ifd.roundtip = roundtip
		local attData = v
		ifd.who = ffd.attid
		ifd.target = attData.id
		ifd.mainType = MAIN_TYPE_FightAfter
		ifd.subType = SUB_TYPE_ResChange
		ifd.resType = attData.result
		ifd.value = attData.att_value
		if attData.result == proto.HPMP_RESULT_TYPE_HRT_ADDHP then
			ifd.param = addHpdelay
			addHpdelay = addHpdelay + 1
		end
		ifd.info = ''
		
		--ǹ
		if attData.result == HPMP_RESULT_TYPE_EX_HRT_DECMP_EX then
			ifd.resType = proto.HPMP_RESULT_TYPE_HRT_DECMP
			ifd.rune_decmp = true
		end
		
		tempifdList[#tempifdList+1] = ifd
	end
	
	local tround = self.m_curFightMsg.tround
	for k, v in ipairs(tempifdList) do
		if v.info ~= '' and v.info then
			v.info = SSFmt('[%u]%s', ffd.round, v.info)
			local round = ffd.round
			v.roundtip = SSFmt(TDGETSTRCS('FIGHT_ROUNT'), tround > round and tround - round or tround)
		end
	end
	self.ifdList = tempifdList
	
	if ffd.round ~= attacker.curRound then
		attacker.curRound = attacker.curRound + 1
		attacker:RunBuff()
		UpdateFightObjectBuffDetails(attacker)
	end
end

function FightMgr:UpdateFightInnerFrame(ifd)
	local battleInfo = self.battleInfo
	if ifd ~= nil then
		if ifd.info ~= '' then
			battleInfo[#battleInfo+1] = ifd.info
		end
		local target = self:GetFod(ifd.target)
		if ifd.subType == SUB_TYPE_ResChange then
			if ifd.resType == proto.HPMP_RESULT_TYPE_HRT_ADDHP then
				target:AddHp(ifd.value)
			elseif ifd.resType == proto.HPMP_RESULT_TYPE_HRT_ADDMP then
				target:AddMp(ifd.value)
			elseif ifd.resType == proto.HPMP_RESULT_TYPE_HRT_DECMP then
				target:DecMp(ifd.value)
			elseif ifd.resType == proto.HPMP_RESULT_TYPE_HRT_DECHP then
				target:DecHp(ifd.value)
			end
		elseif ifd.subType == SUB_TYPE_BuffChange then
			local buffData = {}
			buffData.id = ifd.target
			buffData.fbt = ifd.resType
			buffData.round_cd = ifd.value
			target:AddBuff(buffData)
		end
	else
		local result = self:GetFightResult()
		if result ~= nil then
			self.waitTicks = result.sleepTime
		elseif self.waitTicks > 0 then
			self.waitTicks = self.waitTicks - 1
		end
		
		local colors = {'888888', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}
		if result ~= nil then
			if self.fightType == FIGHT_TYPE_Normal then
				if result.result == proto.FIGHT_RESULT_TYPE_FRT_WIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_RESULT_WIN')
					if result.exp ~= 0 then
						battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_GET_EXP'), result.exp)
					end
					if result.money ~= 0 then
						battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_GET_MONEY'), result.money)
					end
					for k, v in ipairs(result.itemList) do
						local templateIdWithColor = v
						local templateId = templateIdWithColor % 0x1000000
						local isSold = bit.test(templateIdWithColor, 32)
						local ids = itemCfg[templateId]
						local color = math.floor(templateIdWithColor / 0x10000000) % 8
						if color == 0 then
							battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_GET_ITEM'), ids.name, 1)
						else
							local value = SSFmt('<c=%s>%s</>', colors[color], ids.name)
							battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS(isSold and 'BAT_GET_EQUIP_BUT_SOLD' or 'BAT_GET_EQUIP'), value)
						end
					end
					if #result.bossAward > 0 then
						local box = result.bossAward[1]
						local ids = itemCfg[box.id]
						if ids then
							battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_GET_ITEM'), ids.name, box.num)
						end
					end
					for k, v in ipairs(result.boxInfo) do
						local ids = itemCfg[v]
						if ids then
							battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_FINDBOX_OPEN'), ids.name)
						end
					end
				elseif result.result == proto.FIGHT_RESULT_TYPE_FFR_LOSE then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_RESULT_LOSE')
				elseif result.result == proto.FIGHT_RESULT_TYPE_FFR_ENEMYAWAY then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_RESULT_AWAY')
				end
			elseif self.fightType == FIGHT_TYPE_BattleField then
				local UNWIN = 100
				local HDWIN = 101
				local UNHD = 102
				if result.result == UNWIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_BATFIELD_UNWIN')
				elseif result.result == HDWIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_BATFIELD_HDWIN')
				else
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_BATFIELD_HDUN')
				end
			elseif self.fightType == FIGHT_TYPE_EliteMap then
				local ELITEWIN = 200
				local ELITEFALI = 201
				local ELITEPASS = 202
				local ELITENOROUND= 203
				if result.result == ELITEWIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_WIN')
				elseif result.result == ELITEFALI then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_FAIL')
				elseif result.result == ELITENOROUND then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_NOROUND')
				else
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_PASS')
				end
			elseif self.fightType == FIGHT_TYPE_Quarry then
				local WIN = 300
				local LOSE = 301
				if result.result == WIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_WIN')
				elseif result.result == LOSE then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_FAIL')
				end
			elseif self.fightType == FIGHT_TYPE_Abyss then
				local WIN = 400
				local FAIL = 401
				local NOROUND = 402
				if result.result == WIN then
					battleInfo[#battleInfo+1] = TDGETSTRCS('ABYSS_BATTLE_WIN')
				elseif result.result == FAIL then
					battleInfo[#battleInfo+1] = TDGETSTRCS('BAT_ELITE_FAIL')
				elseif result.result == NOROUND then
					battleInfo[#battleInfo+1] = TDGETSTRCS('ABYSS_BATTLE_NOROUND')
				end
			end
		else
			battleInfo[#battleInfo] = nil
		end
		
		if self.fightType == FIGHT_TYPE_Normal then
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_WAIT_FIND'), self.waitTicks)
		elseif self.fightType == FIGHT_TYPE_BattleField then
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_BATFIELD_FIND'), self.waitTicks)
		elseif self.fightType == FIGHT_TYPE_EliteMap then
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_ELITE_WAIT'), self.waitTicks)
		elseif self.fightType == FIGHT_TYPE_Quarry then
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('BAT_ELITE_WAIT'), self.waitTicks)
		elseif self.fightType == FIGHT_TYPE_Abyss then
			battleInfo[#battleInfo+1] = SSFmt(TDGETSTRCS('ABYSS_BATTLE_WAIT'), self.waitTicks)
		end
	end
	
	while #battleInfo > 20 do
		table.remove(battleInfo, 1)
	end
	if self.panel then
		UpdateFightInnerFrame(self, ifd)
		UpdateBattleLog(self)
	end
end

function InitFightObjectDetails(rootWidget, panelName, fod)
	InitFightObjectDetails_(rootWidget, UIHO:seekWidgetByName(rootWidget, panelName), fod, fod and not fod.isLeft and fod.iconid > 10000)
end
function InitFightObjectDetails_(rootWidget, panel, fod, isBoss)
	if panel == nil then return end
	
	if fod == nil then
		panel:setEnabled(false)
		return
	end
	
	fod.panel = panel
	if isBoss then
		panel:setEnabled(false)
		fod.panel = panel:clone()
		panel:getParent():addChild(fod.panel)
		panel = fod.panel
		panel:setName('pnl_boss')
	end
	
	panel:setEnabled(true)
	local label = UIHO:seekWidgetByName(panel, 'lbl_name')
	if label then label:setText(fod.name) end
	local iv = UIHO:seekWidgetByName(panel, 'img_boss')
	if iv then iv:setVisible(isBoss) end
	local iv = UIHO:seekWidgetByName(panel, 'img_icon')
	if iv then
		iv:getVirtualRenderer():setVisible(false)
		local prevIcon = iv:getNodeByTag(-100)
		if prevIcon then iv:removeNode(prevIcon) end
		local race_sex_icon = {
			'img_bg_053.png', 'img_bg_057.png', 'img_bg_054.png',
			'img_bg_056.png', 'img_bg_055.png', 'img_bg_058.png',
		}
		local icon
		if fod.iconid < 50 then
			icon = CPSprite:createWithSpriteFrameName(race_sex_icon[fod.iconid%6 + 1])
			if icon then icon:setScale(iv:getScale() * 1.1) end
		elseif fod.iconid < 100 then
			icon = CPSprite:createWithSpriteFrameName(SSFmt('mc_%u.png', fod.iconid-50))
			if icon then icon:setScale(iv:getScale() * 0.9) end
		else
			local iconid = fod.iconid < 10000 and fod.iconid-100 or fod.iconid-10000
			icon = CPSprite:createWithSpriteFrameName(SSFmt('mon_(%u).png', iconid))
			if icon then icon:setScale(isBoss and panel:getScale() or iv:getScale()) end
		end
		if icon then
			local blender = CPSprite:createWithSpriteFrameName('img_bg_012.png')
			if blender then
				if not blender:getTextureRect().size:equals(CCSizeMake(62, 62)) and
					not blender:getTextureRect().size:equals(CCSizeMake(60, 60)) then
					blender = CPSprite:createWithSpriteFrameName('img_bg_039.png')
				end
				icon:addExtBlender(blender)
			end
			iv:addNode(icon, 1, -100)
		end
	end
	UpdateFightObjectDetails(fod, true)
	UpdateFightObjectBuffDetails(fod)
end

function UpdateFightInnerFrame(fds, ifd)
	local CCPointZero = ccp(0, 0)
	local labelRound = fds.panel:seekWidgetByName('Panel_121:text_hp_0')
	
	local info
	if labelRound then
		if ifd then
			labelRound:setText(ifd.roundtip)
		end
		labelRound:setVisible(ifd ~= nil)
	end
	if ifd ~= nil then
		local who = fds:GetFod(ifd.who)
		local target = fds:GetFod(ifd.target)
		local panel = target.panel
		local size = panel:getSize()
		local ptMid = ccp(size.width/2, size.height/2)
		local subType = ifd.subType
		if subType == SUB_TYPE_ResChange then
			local resType = ifd.resType
			if resType == proto.HPMP_RESULT_TYPE_HRT_ADDHP then
				target:FloatText('num_renew1.png', '+', ifd.value, ifd.param*1+0.2)
			elseif resType == proto.HPMP_RESULT_TYPE_HRT_ADDMP then
				target:FloatText('num_goods2.png', '+', ifd.value)
			elseif resType == proto.HPMP_RESULT_TYPE_HRT_DECMP then
				--Ƿ񹥻
				if ifd.rune_decmp then
					target:FloatText('miss', '', 0,nil,TDGETSTRCS('BAT_FLOAT_ATTDEMP'))
				end
			elseif resType == proto.HPMP_RESULT_TYPE_HRT_DECHP then
				if (ifd.mainType == MAIN_TYPE_Fighting and
					(ifd.param == ATTACK_RESULT_TYPE_ART_CRIT or ifd.rune_crit)) or
					ifd.mainType == MAIN_TYPE_FightAfter then
					target:FloatText('num_myhurt.fnt', '-', ifd.value)
				else
					target:FloatText('num_hurt1.fnt', '-', ifd.value)
				end
				--Ա
				if ifd.rune_crit then
					who:FloatText('miss', '', 0,nil,TDGETSTRCS('BAT_FLOAT_ATTCRIT'))
				end
			elseif resType == ATTACK_RESULT_TYPE_EX_ART_DRB_EX then
				--񵲷
				target:FloatText('miss', '', 0,nil,TDGETSTRCS('BAT_FLOAT_ATTDRB'))
			elseif resType == -1 then
				target:FloatText('miss', '', 0,nil,'MISS')
			elseif resType == -2 then --
				target:FloatText('miss', '', 0,nil,TDGETSTRCS('BAT_FLOAT_ATTMISS'))
			else
				LShowMessage('error in UpdateFightInnerFrame resType = '..resType)
				return
			end
		elseif subType == SUB_TYPE_BuffChange then
			UpdateFightObjectBuffDetails(target)
		end
		
		if ifd.mainType == MAIN_TYPE_FightBefore and
			ifd.subType == SUB_TYPE_ResChange and
			ifd.resType == proto.HPMP_RESULT_TYPE_HRT_DECMP then
			if ifd.param ~= 0 then
				local cfg = skillCfg[ifd.param]
				if cfg then
					if ifd.param == 2 or ifd.param == 17 then
						for k = 1, 3 do
							local pt = ifd.paramPos[k]
							if pt and not pt:equals(CCPointZero) then
								local srt = UIStaticRichText:createWithString(cfg.name)
								srt:getStaticRichText():setColor(ccc3(127, 255, 127))
								srt:setOpacity(0)
								srt:setScale(3)
								srt:setPosition(panel:getParent():convertToNodeSpace(panel:convertToWorldSpace(ptMid)))
								panel:getParent():addChild(srt, 100, ActionTag)
								srt:runAction(actionMgr.create(
									'Sequence',
									{'DelayTime', k},
									{'FadeIn', 0.3},
									{'DelayTime', 0.4},
									{'FadeOut', 0.3}
								))
								srt:runAction(actionMgr.create(
									'Sequence',
									{'DelayTime', k},
									{'ScaleTo', 0.3, 1},
									{'EaseExponentialOut', 'MoveTo', 0.4, pt.x, pt.y},
									{'ScaleTo', 0.3, 3},
									{'CallFunc', function () panel:getParent():removeChild(srt) end}
								))
							end
						end
					else
						for k = 1, 3 do
							local pt = ifd.paramPos[k]
							if pt and not pt:equals(CCPointZero) then
								local srt = UIStaticRichText:createWithString(cfg.name)
								srt:setOpacity(0)
								srt:setScale(3)
								srt:setPosition(panel:getParent():convertToNodeSpace(panel:convertToWorldSpace(ptMid)))
								panel:getParent():addChild(srt, 100, ActionTag)
								srt:runAction(actionMgr.create('FadeIn', 0.3))
								srt:runAction(actionMgr.create(
									'Sequence',
									{'ScaleTo', 0.3, 1},
									{'DelayTime', 0.3},
									{'EaseExponentialIn', 'MoveTo', 0.4, pt.x, pt.y},
									{'CallFunc', function () panel:getParent():removeChild(srt) end}
								))
							end
						end
					end
				end
			end
		elseif ifd.mainType == MAIN_TYPE_Fighting and
			ifd.subType == SUB_TYPE_ResChange then
			local needAni = true
			local param = ifd.param
			if param == proto.ATTACK_RESULT_TYPE_ART_ADDHP or
				param == proto.ATTACK_RESULT_TYPE_ART_ADDMP then
				needAni = false
			end
			-- if param == proto.ATTACK_RESULT_TYPE_ART_MISS or
				-- param == proto.ATTACK_RESULT_TYPE_ART_HIT or
				-- param == proto.ATTACK_RESLUT_TYPE_ART_CRIT or
				-- param == proto.ATTACK_RESULT_TYPE_ART_DECMP or
				-- param == proto.ATTACK_RESULT_TYPE_ART_DECMP_CRIT or 
				-- param == ATTACK_RESULT_TYPE_EX_ART_MISS_EX then
				-- needAni = true
			-- end
			if needAni then
				local iv = UIHO:seekWidgetByName(panel, 'img_icon')
				if iv then
					local spr = CPSprite:create()
					local dst = panel:getParent():convertToNodeSpace(iv:convertToWorldSpace(CCPointZero))
					spr:setPosition(dst)
					spr:runAction(actionMgr.create(
						'Sequence',
						{'Animate', 'p102_%u.png', 'p102.plist', 0.05, 5},
						{'CallFunc', function () tcast(panel:getParent(), 'Widget'):removeNode(spr) end}
					))
					tcast(panel:getParent(), 'Widget'):addNode(spr, 0, ActionTag)
					local iiv = UIHO:seekWidgetByName(who.panel, 'img_icon')
					if iiv then
						local prevIcon = iiv:getNodeByTag(-100)
						if prevIcon and prevIcon:numberOfRunningActions() == 0 then
							prevIcon:runAction(actionMgr.create(
								'Sequence',
								{'MoveTo', 0.1, who.isLeft and 32 or -32, 0},
								{'MoveTo', 0.2, 0, 0}
							))
						end
					end
				end
			end
		end
		
		if ifd.mainType ==MAIN_TYPE_Fighting and
			ifd.subType == SUB_TYPE_Relive then
			local reliveTag = fds:GetFod(ifd.who)
			reliveTag:FloatText('miss', '', 0,nil,TDGETSTRCS('BAT_FLOAT_RELIVE'))
		end
		UpdateFightObjectDetails(target, false)
	end
end

function UpdateFightObjectDetails(fod, isInit)
	if not fod then return end
	local panel = fod.panel
	if not panel then return end
	local bar = UIHO:seekWidgetByName(panel, 'bar_hp')
	if bar then
		bar:setPercent(fod.curHp * 100 / fod.hp, not isInit)
		local label = UIHO:seekWidgetByName(panel, 'bar_hp:text_hp')
		if label then label:setText(SSFmt('%u/%u', fod.curHp, fod.hp)) end
		local iv = UIHO:seekWidgetByName(panel, 'bar_hp:img_dead')
		if iv then iv:setVisible(fod.curHp == 0) end
	end
	if panel:getName() ~= 'pnl_player' then return end
	local bar = UIHO:seekWidgetByName(panel, 'bar_mp')
	if bar then
		local percent = fod.curMp * 100 / fod.mp
		bar:setPercent(percent > 100 and 100 or percent, not isInit) 
	end
end

function UpdateFightObjectBuffDetails(fod)
	if not fod then return end
	local panel = fod.panel
	if not panel then return end
	panel = UIHO:seekWidgetByName(panel, 'pnl_buff')
	if not panel then return end
	panel:removeAllChildren()
	local r, str = fod:GetBuffStr()
	if r then
		panel:addChild(UIStaticRichText:createWithString(str, 0))
	end
end

function UpdateBattleLog(fds)
	local sv = fds.panel:seekWidgetByName('battle_log')
	local infoString = '<s=20>'..fds:GetBattleInfo()
	sv:setDirection(SCROLLVIEW_DIR_HORIZONTAL)
	sv:removeAllChildren()
	local srt = UIStaticRichText:createWithString(infoString, sv:getSize().width - 70, 3)
	srt:getStaticRichText():enableBorder(false)
	srt:getStaticRichText():setAnchorPoint(ccp(0, 0))
	srt:setAnchorPoint(ccp(0, 0))
	srt:setPosition(ccp(4, 20))
	sv:addChild(srt)
	--Ƕ˸ôʧҪԶл
	if fds and fds.panel then
		local result = fds:GetFightResult()
		if result and result.result > 200 and fds.panel.m_isEnd == false then
			fds.panel.m_isEnd = true
		end
		if not result and fds.panel.m_isEnd and fds.waitTicks==0 then
			SceneMain:sharedSceneMain():ChangeState(UILayerBattleShow)
		end
	end
end