local bit = require'bit'
local itemCfg = require'config.item'
local gemCfg = require'config.gem'

local item = {}

item.ItemBase = class('ItemBase')
item.ItemBase.INVALID_TYPE_ID = 127

function item.ItemBase:ctor()
end

function item.ItemBase:Pack(packer, clear)
	mp_pack_uint32(packer, 5)

	mp_pack_uint64(packer, self.ownerId)
	mp_pack_uint32(packer, self.templateId)
	mp_pack_uint64(packer, self.uniqueId)
	mp_pack_uint32(packer, self.stackCount)
	mp_pack_uint32(packer, self.baseFlag)
end

function item.ItemBase:Unpack(data, len, offset)
	local ret
	local __rec_count__ = 0
	ret, offset, __rec_count__ = mp_unpack_int32(data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.ownerId = mp_unpack_uint64(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.templateId = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.uniqueId = mp_unpack_uint64(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.stackCount = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.baseFlag = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	return 0, offset
end

function item.ItemBase:bind()
	return bit.test(self.baseFlag, 1)
end

function item.ItemBase:GetConfig()
	return itemCfg[self.templateId]
end

function item.ItemBase:getColorName()
	local cfg = self:GetConfig()
	if cfg then
		return cfg.name
	end
	return 'null'
end

item.ItemEquipment = class('ItemEquipment', item.ItemBase)
local super = item.ItemEquipment.__super

function item.ItemEquipment:ctor()
	super.ctor(self)
	self.m_jewelProperty = {}
	self.m_randomProperty = {}
	self.m_score = 0
	self.m_scoreex = 0
end

function item.ItemEquipment:Pack(packer, clear)
	super.Pack(self, packer, false)

	mp_pack_uint32(packer, 9);

	mp_pack_uint32(packer, 4);
	for i = 1, 4, 1 do
		mp_pack_uint32(packer, self.m_jewelProperty[i])
	end

	mp_pack_uint32(packer, RANDOM_PROPERTY_TYPE_MAX);
	for i = 1, RANDOM_PROPERTY_TYPE_MAX, 1 do
		mp_pack_uint32(packer, self.m_randomProperty[i])
	end

	mp_pack_uint32(packer, self.m_specialProperty)
	mp_pack_uint32(packer, self.m_enhancedLevel)
	mp_pack_uint32(packer, self.m_2specialProperty)
	mp_pack_uint16(packer, self.m_starNum)
	mp_pack_uint16(packer, self.m_starAttrType)
	mp_pack_uint32(packer, self.m_starAttrValue)
	mp_pack_uint32(packer, self.m_starExp)
end

function item.ItemEquipment:Unpack(data, len, offset)
	local ret
	ret, offset = super.Unpack(self, data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end

	local __rec_count__ = 0
	ret, offset, __rec_count__ = mp_unpack_int32(data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end

	self.m_jewelSlotCount = 0
	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		local count
		ret, offset, count = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end

		for i = 1, count, 1 do
			ret, offset, self.m_jewelProperty[i] = mp_unpack_uint32(data, len, offset)
			if (ret ~= 0) then
				error("unpack error")
			end
			if (self.m_jewelProperty[i] ~= 0) then
				self.m_jewelSlotCount = self.m_jewelSlotCount + 1
			end
		end
	end

    self.m_randomPropertyCount = 0
	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		local count
		ret, offset, count = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end

		for i = 1, count, 1 do
			ret, offset, self.m_randomProperty[i] = mp_unpack_uint32(data, len, offset)
			if (ret ~= 0) then
				error("unpack error")
			end
			if (self.m_randomProperty[i] ~= 0) then
				self.m_randomPropertyCount = self.m_randomPropertyCount + 1
			end
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_specialProperty = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_enhancedLevel = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_2specialProperty = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_starNum = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_starAttrType = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_starAttrValue = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_starExp = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	while (__rec_count__ > 0) do
		__rec_count__ = __rec_count__ - 1
		ret, offset = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end

	return 0, offset
end

function item.ItemEquipment:GetColor()
	if self.m_specialProperty ~= 0 or
		self.m_2specialProperty ~= 0 then
		return 5
	end
	return self.m_randomPropertyCount
end

function item.ItemEquipment:getColorIdx()
	if self.m_specialProperty ~= 0 or
		self.m_2specialProperty ~= 0 then
		return 5
	end
	return self.m_randomPropertyCount
end

function item.ItemEquipment:getStarDesc()
	if self.m_starNum > 0 then
		return SSFmt(TDGETSTRCS('EQUIP_STAR_NAME'), self.m_starNum)
	end
	return ''
end

function item.ItemEquipment:GetSpecialLevel()
	local level = 0
	local expr = self.m_specialProperty
	for k, v in ipairs(g_Player.sqexp) do
		if expr < v then break end
		level = k
		expr = expr - v
	end
	return level, expr
end

function item.ItemEquipment:Get2SpecialLevel()
	local level = 0
	local expr = self.m_2specialProperty
	for k, v in ipairs(g_Player.sq2exp) do
		if expr < v then break end
		level = k
		expr = expr - v
	end
	return level, expr
end

function item.ItemEquipment:getSqType()
	if self.m_specialProperty ~= 0 and
		self.m_2specialProperty ~= 0 then
		return EST_SQ_GOD
	end
	if self.m_specialProperty ~= 0 then
		return EST_SQ_NOR
	end
	if self.m_2specialProperty ~= 0 then
		return EST_SQ_CRIDET
	end
	return EST_SQ_NULL
end

function item.ItemEquipment:getSqDesc()
	if self.m_specialProperty ~= 0 and
		self.m_2specialProperty ~= 0 then
		return TDGETSTRCS('EQUIP_SQ_NAME_'..8)
	end
	if self.m_specialProperty ~= 0 then
		return TDGETSTRCS('EQUIP_SQ_NAME_'..6)
	end
	if self.m_2specialProperty ~= 0 then
		return TDGETSTRCS('EQUIP_SQ_NAME_'..7)
	end
	return ''
end

local colorStr = {'888888', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}
function item.ItemEquipment:getColorName()
	local cfg = self:GetConfig()
	if cfg then
		local nameDesc = self:getStarDesc()..self:getSqDesc()
		if nameDesc ~= '' then nameDesc = '['..nameDesc..']' end
		return SSFmt('<c=%s>%s%s</>', colorStr[self:getColorIdx()+1], cfg.name, nameDesc)
	end
	return 'null'
end

local colorStr = {'888888', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}
function item.ItemEquipment:getColorNameEx()
	local cfg = self:GetConfig()
	if cfg then
		return SSFmt('<c=%s>%s</>', colorStr[self:getColorIdx()+1], cfg.name)
	end
	return 'null'
end

function item.ItemEquipment:GetRandmProperty(_type)
	if _type == 0 or _type > table.getn(self.m_randomProperty) then
		return 0
	else
		return self.m_randomProperty[_type]
	end
end

function item.ItemEquipment:getGemLv()
	local lv = 0x0fffffff
	for i = 1, 4, 1 do
		local cfg = gemCfg[self.m_jewelProperty[i]]
		if cfg == nil then return 0 end
		if cfg.level < lv then lv = cfg.level end
	end
	return lv
end

function item.ItemEquipment:GetScore()
	if self.m_score ~= 0 then return self.m_score, self.m_scoreex end
	local ids = itemCfg[self.templateId]
	if ids == nil then return 1 end
	-- 强化属性只提升固定属性部分
    -- 提升规则为 elevel * 2% * color
	local enhancedLevel = self.m_enhancedLevel * 2 * self.m_randomPropertyCount

	self.m_score = self.m_score + (ids.damageMin + ids.damageMax)
	self.m_score = self.m_score + ids.armor
	self.m_score = self.m_score + ids.accuracy
	self.m_score = self.m_score + ids.dodge
	self.m_score = self.m_score + ids.critical
	self.m_score = self.m_score + ids.hp / 10
	self.m_score = self.m_score + ids.mp / 10
	self.m_score = self.m_score + ids.decCritical
	self.m_score = self.m_score + ids.physicalResistance
	self.m_score = self.m_score + ids.magicResistance
	
	self.m_scoreex = self.m_score * enhancedLevel / 100
	
	for k = 1, 4, 1 do
		self.m_score = self.m_score + self.m_randomProperty[k]
	end
	
	return self.m_score, self.m_scoreex
end

item.ItemMisc = class('ItemMisc', item.ItemBase)
local super = item.ItemMisc.__super

function item.ItemMisc:ctor()
	super.ctor(self)
end

function item.ItemMisc:Pack(packer, clear)
	super.Pack(self, packer, clear)
end

function item.ItemMisc:Unpack(data, len, offset)
	local ret
	ret, offset = super.Unpack(self, data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end

	return 0, offset
end

function item.ItemMisc:getColorName()
	local cfg = self:GetConfig()
	if cfg then
		if self.stackCount == 1 then
			return cfg.name
		end
		return SSFmt('%s*%u', cfg.name, self.stackCount)
	end
	return 'null'
end

item.ItemSupplement = class('ItemSupplement', item.ItemBase)
local super = item.ItemSupplement.__super

EquipRuneColors = {
	{ 255, 255, 255 },
	{ 140, 255, 130 },
	{ 100, 150, 255 },
	{ 255, 0, 255 },
	{ 255, 127, 0 },
	{ 255, 0, 0 },
}

function item.ItemSupplement:ctor()
	super.ctor(self)
	self.m_uiexp = 0
end

function item.ItemSupplement:Pack(packer, clear)
	super.Pack(self, packer,false)
	
	mp_pack_uint32(packer, 1)
	mp_pack_uint32(packer, self.m_uiexp)
end

function item.ItemSupplement:Unpack(data, len, offset)
	local ret
	ret, offset = super.Unpack(self, data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end
	
	local __rec_count__ = 0
	ret, offset, __rec_count__ = mp_unpack_int32(data, len, offset)
	if (ret ~= 0) then
		error("unpack error")
	end
	
	if (__rec_count__ > 0) then
		__rec_count__ = __rec_count__ - 1
		ret, offset, self.m_uiexp = mp_unpack_uint32(data, len, offset)
		if (ret ~= 0) then
			error("unpack error")
		end
	end
	
	return 0, offset
end

function item.ItemSupplement:getColorName()
	local cfg = self:GetConfig()
	if cfg then
		return cfg.name
	end
	return 'null'
end

function item.ItemSupplement:getAttrStr()
	 
end

function item.ItemSupplement:LevelStr()
	 return SSFmt('Lv.%d',self:Level())
end

local colorStrEx = {'ffffff', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}
function item.ItemSupplement:getRuneColorName()
	local lv = self:Level()
	return SSFmt('<c=%s>Lv%d(%s)</>', colorStrEx[lv], lv, string.sub(self:getColorName(), 1, 6))
end

function item.ItemSupplement:Level()
	return g_Player:GetRuneItemLevel(self.m_uiexp)
end

return item