ChargeTurnplateActivity = class('ChargeTurnplateActivity', LuaUI)
local super = ChargeTurnplateActivity.__super

function ChargeTurnplateActivity:init()
	if not super.init(self) then return false end
	
	if not self:initRootWidget('activity_chargeturnplate.json', true) then return false end
	
	self.g_nowPoint = 0	--当前的指针角度
	self.g_point = {0, 45, 90, 135, 180, 225, 270, 315} 	--指针角度
	self.g_runAction = false
	self.awardid = 0
	self.g_ActivityStatus = 0	--0:未开启，1:开启，2:已结束

	local btn_draw = self:seekWidgetByName('btn_draw')
	if btn_draw ~= nil then btn_draw:setTouchEnabled(true) end
	
	local img_pointer = self:seekWidgetByName('img_pointer')
	if img_pointer ~= nil then
		local actions = CCArray:create()
		actions:addObject(CPRotateTo:create(0, self.g_nowPoint))
		img_pointer:runAction(CPSequence:create(actions))
	end
	
	self:addLuaHandler('btn_draw', function ()
		if self.g_runAction then return end
		
		local _btn_draw = self:seekWidgetByName('btn_draw')
		if _btn_draw ~= nil then _btn_draw:setTouchEnabled(false) end
		
		g_SendMsg({ ptype = 'chargeturnplateitems', })
		
		g_SendMsg({ ptype = 'chargeturnplateaward', awardid = self.awardid, })
		
		self.g_runAction = true
	end)
	
	self:addLuaHandler('btn_charge', function ()
		local layer = self:GetRootLayer()
		self:close()
		Charge:Show(layer)
	end)
	
	self:addLuaHandler('btn_rlue', function ()
		LShowMessage(getString('CHARGE_TURNPLATE_ACTIVITY_TIP'), nil, nil, MBT_OK, self._nil, self._nil, self._nil)
	end)
	
	self:addLuaHandler('btn_close', function () self:close() end)
	self:OutClickClose()
	
	g_SendMsg({ ptype = 'chargeturnplate', })
	
	g_SendMsg({ ptype = 'chargeturnplateitems', })
	
	return true
end

function ChargeTurnplateActivity:onEnter()
	self:regJMsgHandler('chargeturnplate_time', self.onTime)
	self:regJMsgHandler('chargeturnplate_info', self.onActivityInfo)
	self:regJMsgHandler('chargeturnplate_items', self.onUpdateItems)
	self:regJMsgHandler('chargeturnplate_award', self.onAction)
	self:regJMsgHandler('chargeturnplate_error', self.onError)
end

function ChargeTurnplateActivity:onTime(msg)
	local label = self:seekWidgetByName('lbl_gold')
	if label ~= nil then label:setText(SSFmt('%u', msg.cristal)) end
	
	local bt = msg.bt
	local et = msg.et
	local nt = msg.nt
	self:updateTime(bt, et, nt)
	schedule(self, function ()
		nt = nt + 1
		self:updateTime(bt, et, nt)
	end, 1)
end

function ChargeTurnplateActivity:onActivityInfo(msg)
	self.awardid = msg.awardid
	
	local lbl_times = self:seekWidgetByName('lbl_times')
	if lbl_times ~= nil then lbl_times:setText(SSFmt('%u', msg.times)) end
end

function ChargeTurnplateActivity:onUpdateItems(msg)
	for k,_item in ipairs(msg.items) do
		local nCristal = _item.unCristal
		local idx =_item.unId 		--所在索引
		
		local img = self:seekWidgetByName(string.format('ImgItem_%d', idx))
		if img ~= nil then
			local effect = img:getChildByTag(9527)
			if effect ~= nil then img:removeChildByTag(9527, true) end
			local label = UIStaticRichText:createWithString(string.format('%d', nCristal), img:getSize().width*2, 3)
			img:addChild(label, 995, 9527)
		end
		
		local itemImg = self:seekWidgetByName(string.format('ImgItem_%d:itemImg', idx))
		if itemImg ~= nil then
			itemImg:setVisible(false)
		end
	end
end

function ChargeTurnplateActivity:onAction(msg)
	local index = msg.index
	if index < 1 then return end
	
	local actions = CCArray:create()
	local img_pointer = self:seekWidgetByName('img_pointer')
	actions:addObject(CPRotateBy:create(0.5, 180))
	actions:addObject(CPRotateBy:create(0.5, 360 + self.g_point[index] - self.g_nowPoint))
	actions:addObject(CPRotateBy:create(0.5, 540))
	actions:addObject(CPRotateBy:create(0.5, 330))
	actions:addObject(CPRotateBy:create(0.5, 190))
	actions:addObject(CPRotateBy:create(0.5, 120))
	actions:addObject(CPRotateBy:create(0.5, 80))
	img_pointer:runAction(CPSequence:create(actions))
	
	performWithDelay(self, function ()
		self.g_runAction = false
		
		local _btn_draw = self:seekWidgetByName('btn_draw')
		if _btn_draw ~= nil then _btn_draw:setTouchEnabled(true) end
		
		local itemImg = self:seekWidgetByName(string.format('ImgItem_%d:itemImg', index))
		if itemImg ~= nil then itemImg:setVisible(true) end
		
		self.g_nowPoint = self.g_point[index]
	end, 3.5)
end

function ChargeTurnplateActivity:onError(msg)
	local index = msg.index
	if index < 0 then
		local _btn_draw = self:seekWidgetByName('btn_draw')
		if _btn_draw ~= nil then _btn_draw:setTouchEnabled(true) end
		
		if index == -2 then
			LShowMessage(getString('CHARGE_TURNPLATE_ACTIVITY_ERROR_2'), nil, nil, MBT_OK, self._nil, self._nil, self._nil)
		elseif index == -3 then
			LShowMessage(getString('CHARGE_TURNPLATE_ACTIVITY_ERROR_1'), nil, nil, MBT_OK, self._nil, self._nil, self._nil)
		elseif index == -4 then
			if self.g_ActivityStatus == 0 then
				LShowMessage(getString('SD_ACTIVITY_NOT_BEGIN'), nil, nil, MBT_OK, self._nil, self._nil, self._nil)
			elseif self.g_ActivityStatus == 2 then
				LShowMessage(getString('SD_ACTIVITY_END'), nil, nil, MBT_OK, self._nil, self._nil, self._nil)
			end
		end
	end
	self.g_runAction = false
end

local function getTime(tm)
	local d = math.floor(tm / 86400)
	tm = tm % 86400
	local h = math.floor(tm / 3600)
	tm = tm % 3600
	local m = math.floor(tm / 60)
	tm = tm % 60
	return d, h, m, tm
end

function ChargeTurnplateActivity:updateTime(bt, et, nt)
	local txt = self:seekWidgetByName('lab_endTime')
	local tip1
	if nt < bt then
		tip1 = string.format(getString('SD_ACTIVITY_BEGIN'), getTime(bt - nt))
		self.g_ActivityStatus = 0
	elseif nt > et then
		tip1 = (getString('SD_ACTIVITY_END'))
		self.g_ActivityStatus = 2
	else
		tip1 = string.format(getString('SD_ACTIVITY_ING'), getTime(et - nt))
		self.g_ActivityStatus = 1
	end
	txt:setText(string.format('%s\n', tip1))
end

function ChargeTurnplateActivity:_nil()
end