local itemCfg = require'config.item'

ChristmasActivity = class('ChristmasActivity', LuaUI)
local super = ChristmasActivity.__super

local colorStr = {'888888', '88ff88', '8888ff', 'ff88ff', 'ff8800', 'ff0000'}
local function getColorName(id, color)
	local cfg = itemCfg[id]
	if not cfg then return 'null' end
	
	if cfg.itemClass ~= ITEM_CLASS_EQUIPMENT then
		return SSFmt(TDGETSTRCS("SD02_MAIL_AWARD_ITEM"), cfg.name, color)
	end
	
	local star = math.floor(color / 100)
	local color = color % 100 + 1
	if color > 8 then color = 8 end
	
	local nameEx = ''
	if star > 0 then
		nameEx = nameEx..SSFmt(TDGETSTRCS('EQUIP_STAR_NAME'), star)
	end
	if color >= 6 and color <= 8 then
		nameEx = nameEx..TDGETSTRCS('EQUIP_SQ_NAME_'..color)
	end
	if nameEx ~= '' then nameEx = '['..nameEx..']' end
	
	if color > 6 then color = 6 end
	return SSFmt('<c=%s>%s%s</>', colorStr[color], cfg.name, nameEx)
end

local function getTime(tm)
	local d = math.floor(tm / 86400)
	tm = tm % 86400
	local h = math.floor(tm / 3600)
	tm = tm % 3600
	local m = math.floor(tm / 60)
	tm = tm % 60
	return d, h, m, tm
end

local awdMoney = {10, 30, 50, 100, 300, 500, 1000, 2000, 3000, 5000}
function ChristmasActivity:init()
	if not super.init(self) then return false end
	if not self:initRootWidget('activity_christmas.json', true) then return false end
	
	for k, v in ipairs(awdMoney) do
		self:seekWidgetByName('btn_'..v):setEnabled(false)
	end
	self:addLuaHandler('btn_rlue', function ()
		LShowMessage(TDGETSTRCS('SD_HELP_22'),
			TDGETSTRCS('SD_HELP_TITLE_22'))
	end)
	self:addLuaHandler('btn_charge', function ()
		local layer = self:GetRootLayer()
		self:close()
		Charge:Show(layer)
	end)
	self:OutClickClose()
	g_SendMsg({ptype = 'christ_lst'})
	
	return true
end

function ChristmasActivity:onEnter()
	self:regJMsgHandler('christ_lst', self.onRecvActList)
	self:regJMsgHandler('christ_del', self.onRecvActDel)
end

function ChristmasActivity:onRecvActList(msg)
	self.charge = msg.charge
	self:seekWidgetByName('lab_changdays'):setText(SSFmt(TDGETSTRCS('SD_CHARGEACTIVITY_TCD1'), msg.charge/10))
	for k, v in ipairs(msg.items) do
		local btn = self:seekWidgetByName('btn_'..v.chargeDay)
		if btn then
			btn:setEnabled(true)
			if v.canGet == 1 then
				btn:addNode(CreateAnimation('sq_%03u.png', 'main-equip.plist', 0.2, 14))
			end
			self:addLuaHandler(btn, function ()
				self:ShowAward(v)
			end)
		end
	end
	local bt, et, nt = msg.bt, msg.et, msg.nt
	self:updateTime(bt, et, nt)
	schedule(self, function ()
		nt = nt + 1
		self:updateTime(bt, et, nt)
	end, 1)
end
	
function ChristmasActivity:updateTime(bt, et, nt)
	local txt = self:seekWidgetByName('lab_endTime')
	local tip1
	if nt < bt then
		tip1 = string.format(getString('SD_ACTIVITY_BEGIN'),
			getTime(bt - nt))
	elseif nt > et then
		tip1 = (getString('SD_ACTIVITY_END'))
	else
		tip1 = string.format(getString('SD_ACTIVITY_ING'),
			getTime(et - nt))
	end
	txt:setText(string.format('%s\n', tip1))
end

function ChristmasActivity:onRecvActDel(money)
	local btn = self:seekWidgetByName('btn_'..money)
	if btn then btn:setEnabled(false) end
end

local function getAwdDesc(t, v, p)
	if t == 0 then
		return SSFmt(TDGETSTRCS('MERC_GOLD_TXT1'), v)
	elseif t == 1 then
		return SSFmt(TDGETSTRCS('MERC_CRIS_TXT1'), v)
	else
		return getColorName(v, p)
	end
end
function ChristmasActivity:ShowAward(info)
	local item = info.itemList[1]
	local tip = {}
	for k, v in ipairs(info.itemList) do
		tip[#tip + 1] = getAwdDesc(v.type, v.value, v.parm)
	end
	LShowMessage(table.concat(tip, '\n'), nil, TDGETSTRCS('CHARGERANK_ACTIVITY_GET'),
		MBT_CANCELCUSTOM, nil, nil,
		function ()
			if self.charge < info.chargeDay then
				LShowMessage(TDGETSTRCS('CHRIST_CHARGE_NOT_ENOUGH'))
			else
				g_SendMsg({ptype = 'christ_awd', money = info.chargeDay})
			end
		end)
end