local tcast = tolua.cast
local itemCfg = require'config.item'

DailyTask = class('DailyTask', LuaUI)
local super = DailyTask.__super

function DailyTask:init()
	if not super.init(self) then return false end
	if not self:initRootWidget('daily.json', true) then
		return false
	end
	
	self:addLuaHandler('btn_close', function () self:close() end)
	self:regJMsgHandler('dt_list', self.onRecvDTList)
	self:regJMsgHandler('dt_del', self.onRecvDTDel)
	g_SendMsg({ptype = 'daily_task'})
	
	return true
end

function DailyTask:onRecvDTList(list)
	local lv = self:seekWidgetByName('scv_list')
	local children = lv:getChildren()
	while children:count() > 0 do
		lv:removeLastItem()
	end
	
	table.sort(list, function (a, b)
		local cmpA = a.curCnt >= a.needCnt
		local cmpB = b.curCnt >= b.needCnt
		if cmpA and not cmpB then return true end
		if not cmpA and cmpB then return false end
		return a.id < b.id
	end)
	
	local widget = LOADWIDGET('daily_item.json')
	for k, v in ipairs(list) do
		local w = widget:clone()
		local desc = SSFmt(TDGETSTRCS('DAILY_TASK_DESC_'..v.id), v.curCnt, v.needCnt)
		UIHO:seekWidgetByName(w, 'ImageTitle'):setText(desc)
		
		local awdCnt = 0
		if awdCnt < 3 and v.money then  -- 
			awdCnt = awdCnt + 1
			local img = UIHO:seekWidgetByName(w, 'ImgItem_'..awdCnt)
			img:setVisible(true)
			UIHO:seekWidgetByName(img, 'itemImg'):loadTexture('img_bg_money.png', UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(img, 'lab_dec'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_MONEY'), v.money))
		end
		if awdCnt < 3 and v.crystal then  -- ʯ
			awdCnt = awdCnt + 1
			local img = UIHO:seekWidgetByName(w, 'ImgItem_'..awdCnt)
			img:setVisible(true)
			UIHO:seekWidgetByName(img, 'itemImg'):loadTexture('img_bg_051.png', UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(img, 'lab_dec'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_CRYSTAL'), v.crystal))
		end
		if awdCnt < 3 and v.credit then  -- 
			awdCnt = awdCnt + 1
			local img = UIHO:seekWidgetByName(w, 'ImgItem_'..awdCnt)
			img:setVisible(true)
			UIHO:seekWidgetByName(img, 'itemImg'):loadTexture('57.png', UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(img, 'lab_dec'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_CREDIT'), v.credit))
		end
		if awdCnt < 3 and v.itemId and v.itemCnt then  -- 
			awdCnt = awdCnt + 1
			local img = UIHO:seekWidgetByName(w, 'ImgItem_'..awdCnt)
			img:setVisible(true)
			local cfg = itemCfg[v.itemId]
			UIHO:seekWidgetByName(img, 'itemImg'):loadTexture(cfg.texture, UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(img, 'lab_dec'):setText(cfg.name..'*'..v.itemCnt)
		end
		for k = awdCnt + 1, 3 do
			UIHO:seekWidgetByName(w, 'ImgItem_'..k):setVisible(false)
		end
		
		local btnGet = UIHO:seekWidgetByName(w, 'btn_click')
		if v.curCnt < v.needCnt then
			btnGet:loadTextures('btn_qwwc.png', 'btn_qwwc.png', 'btn_qwwc.png', UI_TEX_TYPE_PLIST)
		else
			btnGet:loadTextures('btn_lqjl_1.png', 'btn_lqjl_1.png', 'btn_lqjl_1.png', UI_TEX_TYPE_PLIST)
		end
		self:addLuaHandler(btnGet, function () self:onClickTask(v.id, v.curCnt >= v.needCnt) end)
		
		w:setTag(v.id)
		lv:pushBackCustomItem(w)
	end
end

function DailyTask:onRecvDTDel(idx)
	local lv = self:seekWidgetByName('scv_list')
	local children = lv:getChildren()
	for k = 1, children:count() do
		local w = tcast(children:objectAtIndex(k), 'CCNode')
		if w:getTag() == idx then
			lv:removeItem(k)
			break
		end
	end
end

local guideFun = {}
guideFun[1] = function ()
	SceneMain:sharedSceneMain():ChangeState(UILayerEquipShow)
end
guideFun[2] = function ()
	SceneMain:sharedSceneMain():ChangeState(UILayerBattleShow)
	BattleMap:Show(LayerBattle:sharedLayerBattle())
end
guideFun[3] = function ()
	SceneMain:sharedSceneMain():ChangeState(UILayerArenaShow)
end
guideFun[4] = function ()
	RemakeEquipment:Show(LayerRoot:sharedLayerRoot())
end
guideFun[5] = function ()
	SceneMain:sharedSceneMain():ChangeState(UILayerPetShow)
end
guideFun[6] = function ()
	SceneMain:sharedSceneMain():ChangeState(UILayerBattleShow)
end
function DailyTask:onClickTask(id, comp)
	if comp then
		g_SendMsg({ptype = 'daily_task', idx = id})
	else
		self:close()
		guideFun[id]()
	end
end