local json = require'json'
require'uifunc'

local SSFmt = string.format

DoubleActivity = class('DoubleActivity', LuaUI)
local super = DoubleActivity.__super

function DoubleActivity:init()
	if not super.init(self) then return false end
	
	if not self:initRootWidget('activity_chargeX2.json', true) then return false end
	
	self:OutClickClose()
	self:addLuaHandler('btn_close', function () self:close() end)
	self:addLuaHandler('btn_chargeX2', function ()
		local layer = self:GetRootLayer()
		self:close()
		Charge:Show(layer)
	end)
	
	g_SendMsg({ptype = 'doubleact'})
	
	return true
end

function DoubleActivity:onEnter()
	self:regJMsgHandler('doubleact', self.UpdateUI)
	self:regJMsgHandler('doubleaward', self.DeleteUI)
end

function DoubleActivity:UpdateUI(data)
	local listitem = self:seekWidgetByName('listitem')
	while (listitem:getItems():count() > 0) do
		listitem:removeLastItem()
	end
	
	local txt = self:seekWidgetByName('lab_total')
	txt:setText(string.format(getString('SD_CHARGEACTIVITY_TCD1'), data.total))
	
	local widgetTemplate = LOADWIDGET("activity_item.json")
	for k,_item in ipairs(data.items) do
		local wgtclone = widgetTemplate:clone()		
		local canGet = _item.canGet --能否领取
		local chargeDay =_item.chargeDay
		
		wgtclone:setTag(chargeDay)
		
		local ImageTitle = UIHO:seekWidgetByName(wgtclone, 'ImageTitle')
		ImageTitle:setText(string.format(getString('SD_CHARGEACTIVITY_TCD1'), chargeDay))
		
		--取奖励
		local j = 1
		for v,award in  ipairs(_item.itemList) do	
			local image = UIHO:seekWidgetByName(wgtclone, string.format('ImgItem_%d:itemImg', j))
			local lab_dec = UIHO:seekWidgetByName(wgtclone, string.format('ImgItem_%d:lab_dec', j))
			if image ~= nil and lab_dec ~= nil then 
				local dec = LoadAwardImage(award.type, award.value, award.parm, image)
				lab_dec:setText(dec)
			end
			j =j+1
		end
		
		--隐藏其他的
		for n=j,3 do 
			local itemAward = UIHO:seekWidgetByName(wgtclone, string.format('ImgItem_%d',n))
			if itemAward ~= nil then
				itemAward:setEnabled(false)
			end
		end
		
		--设置按钮
		local btn_click = UIHO:seekWidgetByName(wgtclone, 'btn_click')
		
		self:addLuaHandler(btn_click, function ()
				g_SendMsg({
					ptype = 'doubleactaward',
					index = chargeDay,
				})
			end)
		
		btn_click:setTouchEnabled(canGet == 1)
		if (canGet == 1) then
			btn_click:loadTextureDisabled("btn_lqjl_1.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTextureNormal("btn_lqjl_1.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTexturePressed("btn_lqjl_1.png", UI_TEX_TYPE_PLIST)
		elseif (canGet == 2) then
			btn_click:loadTextureDisabled("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTextureNormal("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTexturePressed("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
		else
			btn_click:loadTextureDisabled("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTextureNormal("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
			btn_click:loadTexturePressed("btn_lqjl_2.png", UI_TEX_TYPE_PLIST)
		end
			
		listitem:pushBackCustomItem(wgtclone)
	end
	
	--倒计时
	local bt = data.bt
	local et = data.et
	local nt = data.nt
	self:updateTime(bt, et, nt)
	schedule(self, function ()
		nt = nt + 1
		self:updateTime(bt, et, nt)
	end, 1)
end

function DoubleActivity:DeleteUI(data)
	local listitem = self:seekWidgetByName('listitem')
	for k = 0, listitem:getItems():count() do
		local widget = listitem:getItem(k)
		if widget ~= nil and widget:getTag() == data.index then
			listitem:removeItem(k)
			break
		end
	end
end

local function getTime(tm)
	local d = math.floor(tm / 86400)
	tm = tm % 86400
	local h = math.floor(tm / 3600)
	tm = tm % 3600
	local m = math.floor(tm / 60)
	tm = tm % 60
	return d, h, m, tm
end

function DoubleActivity:updateTime(bt, et, nt)
	local txt = self:seekWidgetByName('lab_endTime')
	local tip1
	if nt < bt then
		tip1 = string.format(getString('SD_ACTIVITY_BEGIN'),
			getTime(bt - nt))
	elseif nt > et then
		tip1 = (getString('SD_ACTIVITY_END'))
	else
		tip1 = string.format(getString('SD_ACTIVITY_ING'),
			getTime(et - nt))
	end
	txt:setText(string.format('%s\n', tip1))
end

function DoubleActivity:_nil()
end