local tcast = tolua.cast
local proto = require'protocol'
local starCfg = require'config.star'
local MakeItemDetail = require'makeitemdetail'

----------------------------------------------------------------
----------------------------------------------------------------
-- ɽ
SQEatEquip = class('SQEatEquip', LuaUI)
local super = SQEatEquip.__super

function SQEatEquip:init(uid)
	if not super.init(self) then return false end
	if not self:initRootWidget('equip-tunshi.json', true) then return false end
	self:seekWidgetByName('btn_help'):setTag(6)
	self:addLuaHandler('btn_help', function () end)
	
	local item = g_Player:GetItem(uid)
	local equip = MAIN_PLAYER:GetItem(uid)
	if not item or (
		item:getColorIdx() < 5 and
		item.m_starNum == 0) then
		return false
	end
	
	self:updateDetail('item_img_main:item', equip)
	self:updateAttrDesc()
	self:addLuaHandler('item_img_main:item',
		function () self:SelectDest() end)
	for k = 1, 6 do
		self:updateDetail('item_img_'..k..':item', nil)
		self:addLuaHandler('item_img_'..k..':item',
			function () self:SelectSource() end)
	end
	self:updateEatCost()
	
	self:addLuaHandler('btn_auto_select', function ()
		self:AutoSelect()
	end)
	self:addLuaHandler('btn_up', function ()
		local iiv = self:seekWidgetByName('item_img_main:itemView')
		if not iiv then return end
		local star = iiv:GetDummyItem().m_starNum + 1
		local tip = SSFmt(TDGETSTRCS('SYNC_EQ_STAR_UP_COST'), starCfg[star].lu_cost)
		LShowMessage(tip, nil, nil, MBT_OKCANCEL, function ()
			g_SendMsg({ptype = 'eq_star_up', to = iiv:GetUniqueId()})
		end)
	end)
	self:addLuaHandler('btn_do', function ()
		local iiv = self:seekWidgetByName('item_img_main:itemView')
		if not iiv then return end
		local uids = {}
		for k = 1, 6 do
			local iiv = self:seekWidgetByName('item_img_'..k..':itemView')
			if iiv then uids[#uids + 1] = iiv:GetUniqueId() end
		end
		if #uids == 0 then return end
		local msg = proto.C2S_SQEatEquip.new()
		msg.targetId = iiv:GetUniqueId()
		msg.uniqueIds = uids
		g_SendMsg(msg)
	end)
	self:addLuaHandler('btn_close', function () self:close() end)
	self:OutClickClose()
	self:regJMsgHandler('eq_eat', self.onRecvEquipEat)
	self:regJMsgHandler('eq_star_up', self.onRecvEquipStarUp)
	
	return true
end

function SQEatEquip:onRecvEquipEat()
	self:updateAttrDesc()
	for k = 1, 6 do
		self:updateDetail('item_img_'..k..':item', nil)
	end
	self:updateEatCost()
end

function SQEatEquip:onRecvEquipStarUp()
	self:updateAttrDesc()
end

function SQEatEquip:updateDetail(name, item)
	local iv = self:seekWidgetByName(name)
	iv:removeAllChildren()
	if not item then
		iv:setVisible(false)
		tcast(iv:getParent(), 'CCNodeRGBA'):setColor(ccc3(255, 255, 255))
	else
		iv:setVisible(true)
		local w = UIItemImageView:create(item, TLT_Null, PACKAGE_TYPE_BACKPACK)
		iv:addChild(w)
		tcast(iv:getParent(), 'CCNodeRGBA'):setColor(item:getColor())
	end
end

function SQEatEquip:updateAttrDesc()
	local iiv = self:seekWidgetByName('item_img_main:itemView')
	self:seekWidgetByName('lbl_lv'):setVisible(false)
	self:seekWidgetByName('lbl_exp'):setVisible(false)
	self:seekWidgetByName('ImageView_135_0'):setVisible(false)
	self:seekWidgetByName('lab_creditLv'):setVisible(false)
	self:seekWidgetByName('lab_creditexp'):setVisible(false)
	self:seekWidgetByName('Image_19'):setVisible(false)
	self:seekWidgetByName('xj_lv'):setVisible(false)
	self:seekWidgetByName('xj_exp'):setVisible(false)
	self:seekWidgetByName('ImageView_135_1'):setVisible(false)
	if not iiv then return end

	local item = g_Player:GetItem(iiv:GetUniqueId())
	
	local sqLv, sqExp = item:GetSpecialLevel(0)
	if sqLv > 0 then
		self:seekWidgetByName('lbl_lv'):setVisible(true)
		self:seekWidgetByName('lbl_exp'):setVisible(true)
		self:seekWidgetByName('ImageView_135_0'):setVisible(true)
		self:seekWidgetByName('lbl_lv'):setText(SSFmt(TDGETSTRCS('SYNC_SQ_LV_NOR'), sqLv))
		if sqLv >= #g_Player.sqexp-1 then
			self:seekWidgetByName('lbl_exp'):setText('EXP : MAX')
			self:seekWidgetByName('expbar'):setPercent(100)
		else
			self:seekWidgetByName('lbl_exp'):setText(SSFmt(TDGETSTRCS('SYNC_SQ_LV_EXP'), sqExp))
			self:seekWidgetByName('expbar'):setPercent(sqExp / g_Player.sqexp[sqLv+1] * 100)
		end
	end
	
	local sq2Lv, sq2Exp = item:Get2SpecialLevel(0)
	if sq2Lv > 0 then
		self:seekWidgetByName('lab_creditLv'):setVisible(true)
		self:seekWidgetByName('lab_creditexp'):setVisible(true)
		self:seekWidgetByName('Image_19'):setVisible(true)
		self:seekWidgetByName('lab_creditLv'):setText(SSFmt(TDGETSTRCS('SYNC_SQ_LV_CRI'), sq2Lv))
		if sq2Lv>= #g_Player.sq2exp-1 then
			self:seekWidgetByName('lab_creditexp'):setText('EXP : MAX')
			self:seekWidgetByName('credit_expbar'):setPercent(100)
		else
			self:seekWidgetByName('lab_creditexp'):setText(SSFmt(TDGETSTRCS('SYNC_SQ_LV_EXP'), sq2Exp))
			self:seekWidgetByName('credit_expbar'):setPercent(sq2Exp / g_Player.sq2exp[sq2Lv+1] * 100)
		end
	end
	
	local starNum, starExp = item.m_starNum, item.m_starExp
	if starNum > 0 then	
		self:seekWidgetByName('xj_lv'):setVisible(true)
		self:seekWidgetByName('xj_exp'):setVisible(true)
		self:seekWidgetByName('ImageView_135_1'):setVisible(true)
		self:seekWidgetByName('xj_lv'):setText(SSFmt(TDGETSTRCS('SYNC_EQ_LV_STAR'), starNum))
		local cfg = starCfg[starNum + 1]
		if not cfg or starExp >= cfg.lu_exp then
			self:seekWidgetByName('xj_exp'):setText('EXP : MAX')
			self:seekWidgetByName('xj_expbar'):setPercent(100)
			self:seekWidgetByName('btn_up'):setEnabled(cfg ~= nil)
		else
			self:seekWidgetByName('xj_exp'):setText(SSFmt(TDGETSTRCS('SYNC_SQ_LV_EXP'), starExp))
			self:seekWidgetByName('xj_expbar'):setPercent(starExp / cfg.lu_exp * 100)
			self:seekWidgetByName('btn_up'):setEnabled(false)
		end
	else
		self:seekWidgetByName('btn_up'):setEnabled(false)
	end
end

function SQEatEquip:updateEatCost()
	local expSq, expSq2, expStar = 0, 0, 0
	for k = 1, 6 do
		local iiv = self:seekWidgetByName('item_img_'..k..':itemView')
		if iiv then
			local item = g_Player:GetItem(iiv:GetUniqueId())
			expSq = expSq + item.m_specialProperty
			expSq2 = expSq2 + item.m_2specialProperty
			if item.m_starNum > 0 then
				local cfg = starCfg[item.m_starNum]
				if cfg then
					expStar = expStar + cfg.eat_exp + item.m_starExp
				end
			end
		end
	end
	local money = expSq*g_Player.sqeat + expSq2*g_Player.sq2eat + expStar*g_Player.stareat
	if money == 0 then
		self:seekWidgetByName('lbl_coin'):setText(TDGETSTRCS('SD_EatTips'))
	else
		self:seekWidgetByName('lbl_coin'):setText(SSFmt(TDGETSTRCS('SD_MONEY_NEED'), money, g_Player.baseData.money))
	end
end

function SQEatEquip:SelectDest()
	SelectEquipmentForSQEat:Show(self:GetRootLayer(), 0, {},
		function (items)
			if #items == 0 then return end
			self:updateDetail('item_img_main:item', items[1])
		end)
end

function SQEatEquip:SelectSource()
	local iiv = self:seekWidgetByName('item_img_main:itemView')
	if not iiv then return end
	local uids = {}
	for k = 1, 6 do
		local iiv = self:seekWidgetByName('item_img_'..k..':itemView')
		if iiv then uids[#uids + 1] = iiv:GetUniqueId() end
	end
	SelectEquipmentForSQEat:Show(self:GetRootLayer(), iiv:GetUniqueId(), uids,
		function (items)
			for k, v in ipairs(items) do
				if k > 6 then break end
				self:updateDetail('item_img_'..k..':item', v)
			end
			for k = #items + 1, 6 do
				self:updateDetail('item_img_'..k..':item', nil)
			end
			self:updateEatCost()
		end)
end

function SQEatEquip:AutoSelect()
	local iiv = self:seekWidgetByName('item_img_main:itemView')
	if not iiv then return end
	
	local uid = iiv:GetUniqueId()
	local equip = g_Player:GetItem(uid)
	if not equip then return false end
	if equip:GetColor() < 5 and
		equip.m_starNum == 0 then
		return false
	end
	
	-- ɸѡ
	local sqLv = equip:GetSpecialLevel(0)
	local sq2Lv = equip:Get2SpecialLevel(0)
	local sqTp = equip:getSqType()
	if sqTp == EST_SQ_NOR then
		if sqLv >= #g_Player.sqexp-1 then
			sqTp = EST_SQ_NULL
		end
	elseif sqTp == EST_SQ_CRIDET then
		if sq2Lv >= #g_Player.sq2exp-1 then
			sqTp = EST_SQ_NULL
		end
	elseif sqTp == EST_SQ_GOD then
		if sqLv >= #g_Player.sqexp-1 and sq2Lv >= #g_Player.sq2exp-1 then
			sqTp = EST_SQ_NULL
		elseif sqLv >= #g_Player.sqexp-1 then
			sqTp = EST_SQ_CRIDET
		elseif sq2Lv >= #g_Player.sq2exp-1 then
			sqTp = EST_SQ_NOR
		end
	end
	local checkColor
	if sqTp == EST_SQ_GOD then
		-- ˫ж
	elseif sqTp == EST_SQ_NULL then
		-- 
		checkColor = function (equip)
			return equip:getSqType() == EST_SQ_NULL
		end
	else
		-- 
		checkColor = function (equip)
			local tp = equip:getSqType()
			return tp == sqTp or
				tp == EST_SQ_NULL
		end
	end
	-- װɸѡ
	local starNum = equip.m_starNum
	if starNum > 0 then
		local cfg = starCfg[starNum+1]
		if not cfg or equip.m_starExp >= cfg.lu_exp then
			starNum = 0
		end
	end
	local checkStar
	if starNum == 0 then
		-- װ
		checkStar = function (equip)
			return equip.m_starNum == 0
		end
	end
	
	local checkEquip = function (equip)
		if equip:bind() then
			return false
		end
		if equip:getColorIdx() < 5 and
			equip.m_starNum == 0 then
			return false
		end
		if checkColor and not checkColor(equip) then
			return false
		end
		if checkStar and not checkStar(equip) then
			return false
		end
		return true
	end
	
	local idx = 0
	for k, v in pairs(g_Player.backpack) do
		if k ~= uid and checkEquip(v) then
			local equip = MAIN_PLAYER:GetItem(k)
			if equip then
				idx = idx + 1
				self:updateDetail('item_img_'..idx..':item', equip)
				if idx >= 6 then break end
			end
		end
	end
	for k = idx + 1, 6 do
		self:updateDetail('item_img_'..k..':item', nil)
	end
	self:updateEatCost()
end

----------------------------------------------------------------
----------------------------------------------------------------
-- ѡ
SelectEquipmentForSQEat = class('SelectEquipmentForSQEat', LuaUI)
local super = SelectEquipmentForSQEat.__super

function SelectEquipmentForSQEat:init(uid, uids, cb)
	if not super.init(self) then return false end
	if not self:initRootWidget('bag-select.json', true) then return false end
	
	self:seekWidgetByName('btn_all'):setEnabled(false)
	self:seekWidgetByName('btn_white'):setEnabled(false)
	self:seekWidgetByName('btn_green'):setEnabled(false)
	self:seekWidgetByName('btn_blue'):setEnabled(false)
	self:seekWidgetByName('btn_purple'):setEnabled(false)
	self:seekWidgetByName('btn_orange'):setEnabled(false)
	
	local pnlHead = self:seekWidgetByName('pnl_head')
	local pnlBody = self:seekWidgetByName('pnl_body')
	local lv = self:seekWidgetByName('equip_list')
	pnlHead:setVisible(false)
	local sz = pnlHead:getSize()
	local sz1 = pnlBody:getSize()
	pnlBody:setSize(CCSizeMake(sz1.width, sz1.height + sz.height + 60))
	local sz2 = lv:getSize()
	lv:setSize(CCSizeMake(sz2.width, sz2.height + sz.height + 60))
	lv:setDirection(SCROLLVIEW_DIR_VERTICAL)
	lv:removeAllItems()
	
	local checkEquip, maxSel
	if uid == 0 then
		maxSel = 1
		checkEquip = function (equip)
			return equip:GetColor() >= 5 or
				equip.m_starNum > 0
		end
	else
		maxSel = 6
		local equip = g_Player:GetItem(uid)
		if not equip then return false end
		if equip:GetColor() < 5 and
			equip.m_starNum == 0 then
			return false
		end
		
		-- ɸѡ
		local sqLv = equip:GetSpecialLevel(0)
		local sq2Lv = equip:Get2SpecialLevel(0)
		local sqTp = equip:getSqType()
		if sqTp == EST_SQ_NOR then
			if sqLv >= #g_Player.sqexp-1 then
				sqTp = EST_SQ_NULL
			end
		elseif sqTp == EST_SQ_CRIDET then
			if sq2Lv >= #g_Player.sq2exp-1 then
				sqTp = EST_SQ_NULL
			end
		elseif sqTp == EST_SQ_GOD then
			if sqLv >= #g_Player.sqexp-1 and sq2Lv >= #g_Player.sq2exp-1 then
				sqTp = EST_SQ_NULL
			elseif sqLv >= #g_Player.sqexp-1 then
				sqTp = EST_SQ_CRIDET
			elseif sq2Lv >= #g_Player.sq2exp-1 then
				sqTp = EST_SQ_NOR
			end
		end
		local checkColor
		if sqTp == EST_SQ_GOD then
			-- ˫ж
		elseif sqTp == EST_SQ_NULL then
			-- 
			checkColor = function (equip)
				return equip:getSqType() == EST_SQ_NULL
			end
		else
			-- 
			checkColor = function (equip)
				local tp = equip:getSqType()
				return tp == sqTp or
					tp == EST_SQ_NULL
			end
		end
		-- װɸѡ
		local starNum = equip.m_starNum
		if starNum > 0 then
			local cfg = starCfg[starNum+1]
			if not cfg or equip.m_starExp >= cfg.lu_exp then
				starNum = 0
			end
		end
		local checkStar
		if starNum == 0 then
			-- װ
			checkStar = function (equip)
				return equip.m_starNum == 0
			end
		end
		
		checkEquip = function (equip)
			if equip:bind() then
				return false
			end
			if equip:getColorIdx() < 5 and
				equip.m_starNum == 0 then
				return false
			end
			if checkColor and not checkColor(equip) then
				return false
			end
			if checkStar and not checkStar(equip) then
				return false
			end
			return true
		end
	end
	local allItem, allChkBox, curSel = {}, {}, 0
	local function addEquip (k, equip)
		if k == uid then return end
		if not checkEquip(equip) then return end
		local item = MAIN_PLAYER:GetItem(k)
		local w = UIItemImageView:createDetail(item, TLT_RemakeSelect, PACKAGE_TYPE_BACKPACK)
		local iiv = UIHO:seekWidgetByName(w, 'itemView')
		if iiv ~= nil then 
			MakeItemDetail:ChangelScore(UIHO:seekWidgetByName(w, 'pnl_equip_props'), iiv:GetUniqueId())
		end
		local c = UIHO:seekWidgetByName(w, 'chk_box')
		for _, v in ipairs(uids) do
			if v == k then
				c:setSelectedState(true)
				curSel = curSel + 1
				break
			end
		end
		self:addLuaHandler(w, function ()
			if c:isEnabled() then
				local chk = not c:getSelectedState()
				c:setSelectedState(chk)
				if chk then curSel = curSel + 1
				else curSel = curSel - 1 end
				for k, v in ipairs(allChkBox) do
					v:setEnabled(curSel < maxSel or v:getSelectedState())
				end
			end
		end)
		c:setTouchEnabled(false)
		lv:pushBackCustomItem(w)
		allItem[#allItem + 1] = UIHO:seekWidgetByName(w, 'item_img:itemView')
		allChkBox[#allChkBox + 1] = c
	end
	for k, v in pairs(g_Player.backpack) do
		addEquip(k, v)
	end
	for k, v in ipairs(allChkBox) do
		v:setEnabled(curSel < maxSel or v:getSelectedState())
	end
	self:seekWidgetByName('lbl_message'):setVisible(#allChkBox == 0)
	
	self:addLuaHandler('btn_close', function ()
		local items = {}
		for k, v in ipairs(allChkBox) do
			if v:getSelectedState() then
				items[#items + 1] = allItem[k]:GetDummyItem()
			end
		end
		self:close()
		self.cb(items)
	end)
	self:OutClickClose()
	
	self.cb = cb
	
	return true
end