local itemCfg = require'config.item'

local tcast = tolua.cast
local proto = require'protocol'
local MakeItemDetail = require'makeitemdetail'

local SSFmt = string.format

EnhanceEquip = class('EnhanceEquip', LuaUI)
local super = EnhanceEquip.__super

function EnhanceEquip:init(uid)
	if not super.init(self) then return false end
	if not self:initRootWidget('equip-qianghua.json', true) then return false end
	self:seekWidgetByName('btn_help'):setTag(9)
	self:addLuaHandler('btn_help', function () end)
	
	self.isEquipedItem = false
	self.isPropRefreshed = false
	
	self:addLuaHandler('btn_close', function () 
		if self.isEquipedItem and self.isPropRefreshed then
			-- 请求服务器应用装备属性
			local msg = proto.C2S_RefreshEquipProperty.new()
			msg.uniqueId = 0
			g_SendMsg(msg)
		end
		self:close() 
	end)
	self:addLuaHandler('btn_enforce', function () 
		self.isPropRefreshed = true
		local msg = proto.C2S_EnhanceEquip.new()
		msg.uniqueId = self.uniqueId
		g_SendMsg(msg)
	end)
	
	local item = g_Player:GetItem(uid)
	if item ~= nil then 
		self.isEquipedItem = true
		local ids = itemCfg[item.templateId]
		if ids ~= nil then
			if ids.itemClass ~= ITEM_CLASS_EQUIPMENT then return false end
		else
			return false
		end
		self:updateDetail(item)
	else 
		return false
	end
	
	self:OutClickClose()
	
	self.uniqueId = uid
	self.cfglevel = 0
	self.rpcount = 0
	self.enlevel = 0
	
	return true
end

function EnhanceEquip:OnItemUpdate(_type, where, uniqueId, item, prevItem )
	if self:GetRootLayer() == nil then return end

	if uniqueId == self.uniqueId then
		if _type == IUEVENT_REMOVE then
			self:close()
		elseif _type == IUEVENT_UPDATE then
			self.isPropRefreshed = true
			self:updateDetail(item)
		end
	end
end

function EnhanceEquip:updateDetail(equip)
	if equip == nil then return end
	
	local _rpcount = self.cfglevel
	local _cfglevel = self.rpcount
	local _enlevel = self.enlevel
	
	local ids = itemCfg[equip.templateId]
	if ids == nil then return end

	self.cfglevel = ids.level
	self.rpcount = 4 < equip:getColorIdx() and 4 or equip:getColorIdx()
	self.enlevel = equip.m_enhancedLevel
	
	if self.cfglevel ~= _cfglevel or self.rpcount ~= _rpcount or self.enlevel ~= _enlevel then
		MakeItemDetail:ItemDetail(equip, self:seekWidgetByName("pnl_equip_props"), nil, self:seekWidgetByName("item_img"), 2, 0, -8, true)
		self:updateRes()
	end
end

function EnhanceEquip:updateRes()
	local moneyUse = self.cfglevel * ( 1000 + self.rpcount * 200 )
	local enhanceUseCount = math.floor(self.rpcount + 1 + self.cfglevel / 5)
	if self.enlevel ~= 0 then
		for i = 0, self.enlevel - 1, 1 do
			moneyUse = math.floor(moneyUse * 140 / 100)
			enhanceUseCount = math.floor(enhanceUseCount * 160 / 100)
		end
	end
	
	moneyUse = moneyUse > 0xffffffff and 400000000 or moneyUse
	enhanceUseCount = enhanceUseCount > 0xffffffff and 400000000 or enhanceUseCount
	
	local l = self:seekWidgetByName("lbl_msg")
	if l ~= nil then
		l:setText(SSFmt(TDGETSTRCS("SD_ENHANCE_TIP_0"), (1 + self.rpcount) * 2))
	end

	local ll = self:seekWidgetByName("lbl_jinghua")
	if ll ~= nil then
		ll:setText(SSFmt(TDGETSTRCS("SD_ENHANCE_TIP_1"), enhanceUseCount, g_Player:GetItemCount( 1 )))
	end

	local lll = self:seekWidgetByName("lbl_coin")
	if lll ~= nil then
		lll:setText(SSFmt(TDGETSTRCS("SD_MONEY_NEED"), moneyUse, PLAYER_BASE_DATA.money))
	end
end
