local proto = require'protocol'
local itemCfg = require'config.item'
local MakeItemDetail = require'makeitemdetail'
local tcast = tolua.cast
local SSFmt = string.format

RefreshEquip = class('RefreshEquip', LuaUI)
local super = RefreshEquip.__super

function RefreshEquip:init(uid)
	if not super.init(self) then return false end
	if not self:initRootWidget('equip-xilian.json', true) then return false end
	self:seekWidgetByName('btn_help'):setTag(8)
	self:addLuaHandler('btn_help', function () end)
	
	self.mercid = 0
	self.isEquipedItem = false
	self.isPropRefreshed = false
	
	local lbl = self:seekWidgetByName('lbl_tips')
	if lbl ~= nil then 
		lbl:setText(TDGETSTRCS('EQUIP_MAIN_ATTRIBUTE_'..PLAYER_BASE_DATA.race))
	end
	
	self:addLuaHandler('btn_close', function () 
		if self.isEquipedItem and self.isPropRefreshed then
			-- 请求服务器应用装备属性
			local msg = proto.C2S_RefreshEquipProperty.new()
			msg.uniqueId = 0
			g_SendMsg(msg)
		end
		self:close() 
	end)
	
	self:addLuaHandler('btn_enforce', function () 
		self.isPropRefreshed = true
		local msg = proto.C2S_RefreshEquipProperty.new()
		msg.uniqueId = self.uniqueId
		g_SendMsg(msg)
	end)
	self:addLuaHandler('btn_enforce_yb', function () 
		self.isPropRefreshed = true
		g_SendMsg({ptype='xilian_cristal', uniqueId=self.uniqueId})
	end)
	
	local item = g_Player:GetItem(uid)
	if item ~= nil then 
		self.isEquipedItem = true
		local ids = itemCfg[item.templateId]
		if ids ~= nil then
			if ids.itemClass ~= ITEM_CLASS_EQUIPMENT then return false end
		else
			return false
		end
		self:updateDetail(item)
	else 
		return false
	end
	
	self:OutClickClose()
	
	self.uniqueId = uid
	self.moneyUse = 0
	self.cristalUse = 0
	
	g_SendMsg({ptype='xilian_ask', uniqueId=self.uniqueId})
	
	return true
end

function RefreshEquip:onEnter()
	self:regJMsgHandler('xilian_ask', self.UpdateUI)
	return true
end

function RefreshEquip:OnItemUpdate(_type, where, uniqueId, item, prevItem )
	if self:GetRootLayer() == nil then return end
	
	if uniqueId == self.uniqueId then
		if _type == IUEVENT_REMOVE then
			self:close()
		elseif _type == IUEVENT_UPDATE then
			self:updateDetail(item)
		end
	end
end

function RefreshEquip:OnPropUpdate(tp, pre, cur)
	if tp == SYNC_PROP_TYPE_money or tp == SYNC_PROP_TYPE_cristal then
		self:updateMoney()
	end
end

function RefreshEquip:UpdateUI(msg)
	self.moneyUse = msg.needmoney
	self.cristalUse = msg.needcristal
	self:updateMoney()
end

function RefreshEquip:updateDetail(equip)
	if equip == nil then return end
	MakeItemDetail:ItemDetail(equip, self:seekWidgetByName("pnl_equip_props"), nil, self:seekWidgetByName("item_img"), 2, 0, -8, true)
end

function RefreshEquip:updateMoney()
	local l = self:seekWidgetByName("lbl_xilian")
	if l ~= nil then
		if self.moneyUse > 0 then
			l:setText(SSFmt(TDGETSTRCS("XILIAN_ALL_NEED"), self.moneyUse, PLAYER_BASE_DATA.money, self.cristalUse, PLAYER_BASE_DATA.cristal))
		else
			l:setText(TDGETSTRCS("XILIAN_NEED_ASK"))
		end
	end
end