local type = type

local heroHead = {
	'hero_0.jpg', 'hero_3.jpg', 'hero_2.jpg',
	'hero_1.jpg', 'hero_4.jpg', 'hero_5.jpg',
}
local statusImg = {'icon_txt_kcz.png', 'icon_txt_kcgr.png', 'icon_txt_zyr.png'}

local function getTickStr(tick)
	tick = math.floor(tick)
	return SSFmt(TDGETSTRCS('SD_Time_HMS'),
		tick / 3600, tick % 3600 / 60, tick % 60)
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 矿场界面
DlgQuarry = class('DlgQuarry', LuaUI)
local super = DlgQuarry.__super

local selfIcon = {'Panel_56:Label_44:img_icon', 'Panel_56:Label_45:img_icon'}
local selfTime = {'Panel_56:leb_time1', 'Panel_56:leb_time2'}
local selfTick = {'Panel_56:leb_time1_1', 'Panel_56:leb_time2_2'}
local otherIcon = {'Panel_57:Label_44:img_icon', 'Panel_57:Label_45:img_icon'}
local otherTime = {'Panel_57:leb_time3', 'Panel_57:leb_time4'}
local otherTick = {'Panel_57:leb_time3_1', 'Panel_57:leb_time4_1'}

function DlgQuarry:init(msg)
	if not msg then return false end
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_other.json', true) then return false end
	
	self:scheduleUpdateWithPriorityLua(function (d) self:update(d) end, 0)
	
	self:initSelfQuarry()
	self:initOtherQuarry()
	self:onRecvSyncQuarry(msg)
	
	local btn_help = self:seekWidgetByName("btn_help2")
	local effectSpr = CreateAnimation('new_%02u.png', 'new.plist', 0.1, 10)
	btn_help:addNode(effectSpr, 10, -100)
	local actions = CCArray:create()
	actions:addObject(CPScaleTo:create(0.9,0.9))
	actions:addObject(CPScaleTo:create(0.9,1.2))
	btn_help:runAction(CPRepeatForever:create(CPSequence:create(actions)))
	
	
	self:addLuaHandler('btn_help2', function () LShowMessage(
		TDGETSTRCS('SD_QUARRY_HELP_TITLE'), TDGETSTRCS('SD_QUARRY_HELP_CONTENT')
	) end)
	self:addLuaHandler('btn_close', function () self:close() end)
	self:OutClickClose()
	
	self:regJMsgHandler('miner_ntf', function ()
		g_SendMsg({ptype='quarry_sync'})
	end)
	self:regJMsgHandler('quarry_sync', self.onRecvSyncQuarry)
	self:regJMsgHandler('miner_cap', function (self, msg)
		if type(msg) == 'table' and not IsLuaUIShow('DlgMinerList') then
			DlgMinerList:Show(self:GetRootLayer(), msg, self.data.info.capcost)
		end
	end)
	self:regJMsgHandler('miner_rsc', function (self, msg)
		if type(msg) == 'table' and not IsLuaUIShow('DlgRescueList') then
			DlgRescueList:Show(self:GetRootLayer(), msg)
		end
	end)
	self:regJMsgHandler('miner_avg', function (self, msg)
		if type(msg) == 'table' and not IsLuaUIShow('DlgAvengeList') then
			DlgAvengeList:Show(self:GetRootLayer(), msg)
		end
	end)
	self:regJMsgHandler('miner_sos', function (self, msg)
		if type(msg) == 'table' and not IsLuaUIShow('DlgSOSList') then
			DlgSOSList:Show(self:GetRootLayer(), msg)
		end
	end)
	
	return true
end

function DlgQuarry:initSelfQuarry()
	self:addLuaHandler('Panel_56:btn_buy', function ()
		if self.data.info.buycost == 0 then
			LShowMessage(TDGETSTRCS('SD_QUARRY_NO_BUY_TIP'))
		else
			LShowMessage(SSFmt(TDGETSTRCS('SD_QUARRY_BUY_TIP'), self.data.info.buycost),
			nil, nil, MBT_OKCANCEL,
			function ()
				g_SendMsg({ptype = 'miner_cap_buy'})
			end)
		end
	end)
	self:addLuaHandler('Panel_56:Image_17', function ()
		self:onClickSelfMiner(1)
	end)
	self:addLuaHandler('Panel_56:Image_18', function ()
		self:onClickSelfMiner(2)
	end)
	self:addLuaHandler('Panel_56:btn_change', function ()
		self:ShowQuarry(false)
	end)
	self:addLuaHandler('Panel_56:btn_rescue', function ()
		g_SendMsg({ptype = 'miner_rsc'})
	end)
	self:addLuaHandler('Panel_56:btn_enemy', function ()
		g_SendMsg({ptype = 'miner_avg'})
	end)
end

function DlgQuarry:initOtherQuarry()
	self:addLuaHandler('Panel_57:btn_save', function ()
		g_SendMsg({ptype = 'miner_sos'})
	end)
	self:addLuaHandler('Panel_57:btn_fight', function ()
		g_SendMsg({ptype = 'miner_rvt', union = false})
	end)
	self:addLuaHandler('Panel_57:btn_twofight', function ()
		g_SendMsg({ptype = 'miner_rvt', union = true})
	end)
	self:addLuaHandler('Panel_57:btn_run', function ()
		LShowMessage(SSFmt(TDGETSTRCS('SD_QUARRY_ESC_TIP'), self.data.info.esccost),
			nil, nil, MBT_OKCANCEL,
			function ()
				g_SendMsg({ptype = 'miner_esc'})
			end)
	end)
	self:addLuaHandler('Panel_57:btn_back', function ()
		self:ShowQuarry(true)
	end)
	self:addLuaHandler('Panel_57:btn_look', function ()
		DlgMasterInfo:Show(self:GetRootLayer(), self.data.master)
	end)
end

function DlgQuarry:update(d)
	local data = self.data
	for k = 1, 2 do
		local miner = data.self[k]
		if miner and miner.tick > 0 then
			if miner.tick > d then
				miner.tick = miner.tick - d
			else
				miner.tick = 0
			end
			self:seekWidgetByName(selfTick[k]):setText(getTickStr(miner.tick))
		end
	end
	for k = 1, 2 do
		local miner = data.other[k]
		if miner and miner.tick > 0 then
			if miner.tick > d then
				miner.tick = miner.tick - d
			else
				miner.tick = 0
			end
			self:seekWidgetByName(otherTick[k]):setText(getTickStr(miner.tick))
		end
	end
end

function DlgQuarry:onClickSelfMiner(k)
	local miner = self.data.self[k]
	if not miner then
		g_SendMsg({ptype = 'miner_cap'})
	else
		DlgMinerInfo:Show(self:GetRootLayer(), miner)
	end
end

function DlgQuarry:ShowQuarry(isSelf)
	self:seekWidgetByName('Panel_56'):setVisible(isSelf)
	self:seekWidgetByName('Panel_56'):setEnabled(isSelf)
	self:seekWidgetByName('Panel_57'):setVisible(not isSelf)
	self:seekWidgetByName('Panel_57'):setEnabled(not isSelf)
end

function DlgQuarry:onRecvSyncQuarry(msg)
	self.data = msg
	
	self:seekWidgetByName('Panel_56:btn_change'):setVisible(msg.info.status == 2)
	self:seekWidgetByName('Panel_56:btn_change'):setEnabled(msg.info.status == 2)
	self:seekWidgetByName('Panel_56:btn_change'):setTouchEnabled(msg.info.status == 2)
	self:seekWidgetByName('identity'):loadTexture(statusImg[msg.info.status], UI_TEX_TYPE_PLIST)
	
	self:seekWidgetByName('Panel_56:leb_arrest_number'):setText(tostring(msg.info.curcap))
	self:seekWidgetByName('Panel_56:leb_rescue_number'):setText(tostring(msg.info.currsc))
	self:seekWidgetByName('Panel_56:leb_revenge_number'):setText(tostring(msg.info.curavg))
	
	self:seekWidgetByName('Panel_57:leb_save_number'):setText(tostring(msg.info.cursos))
	self:seekWidgetByName('Panel_57:leb_fight_number'):setText(tostring(msg.info.currvt))
	self:seekWidgetByName('Panel_57:leb_twofitht_number'):setText(tostring(msg.info.currvtu))
	
	for k = 1, 2 do
		local miner = msg.self[k]
		if not miner then
			self:seekWidgetByName(selfIcon[k]):setVisible(false)
			self:seekWidgetByName(selfTime[k]):setVisible(false)
		else
			local imgIcon = self:seekWidgetByName(selfIcon[k])
			imgIcon:setVisible(true)
			imgIcon:setScale(1.35)
			imgIcon:loadTexture(heroHead[miner.race+1], UI_TEX_TYPE_PLIST)
			self:seekWidgetByName(selfTime[k]):setVisible(true)
		end
	end
	local isRevolt = false
	for k = 1, 2 do
		local miner = msg.other[k]
		if not miner then
			self:seekWidgetByName(otherIcon[k]):setVisible(false)
			self:seekWidgetByName(otherTime[k]):setVisible(false)
		else
			local imgIcon = self:seekWidgetByName(otherIcon[k])
			imgIcon:setVisible(true)
			imgIcon:setScale(1.35)
			imgIcon:loadTexture(heroHead[miner.race+1], UI_TEX_TYPE_PLIST)
			self:seekWidgetByName(otherTime[k]):setVisible(true)
			if miner.rvt and miner.uid == tostring(g_Player.baseData.roleId) then
				isRevolt = true
			end
		end
	end
	self:seekWidgetByName('Panel_57:leb_agree'):setVisible(isRevolt)
	self:ShowQuarry(msg.info.status ~= 2)
end


----------------------------------------------------------------
----------------------------------------------------------------
-- 主人详情
DlgMasterInfo = class('DlgMasterInfo', LuaUI)
local super = DlgMasterInfo.__super

function DlgMasterInfo:init(master)
	if not master then return false end
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_master.json', true) then return false end
	
	local imgIcon = self:seekWidgetByName('img_icon')
	imgIcon:setVisible(true)
	imgIcon:setScale(1.35)
	imgIcon:loadTexture(heroHead[master.race+1], UI_TEX_TYPE_PLIST)
	self:seekWidgetByName('leb_name'):setText(master.name)
	self:seekWidgetByName('leb_identity'):setText(tostring(master.level))
	self:seekWidgetByName('leb_faction'):setText(master.faction or TDGETSTRCS('PLAYER_NO_FUNCTION'))
	self:OutClickClose()
	
	return true
end


----------------------------------------------------------------
----------------------------------------------------------------
-- 矿工详情
DlgMinerInfo = class('DlgMinerInfo', LuaUI)
local super = DlgMinerInfo.__super

function DlgMinerInfo:init(miner)
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_message.json', true) then return false end
	
	local imgIcon = self:seekWidgetByName('img_icon')
	imgIcon:setVisible(true)
	imgIcon:setScale(1.35)
	imgIcon:loadTexture(heroHead[miner.race+1], UI_TEX_TYPE_PLIST)
	self:seekWidgetByName('leb_name'):setText(miner.name)
	self:seekWidgetByName('leb_faction'):setText(miner.faction or TDGETSTRCS('PLAYER_NO_FUNCTION'))
	
	local tick = miner.tick
	self:scheduleUpdateWithPriorityLua(function (d)
		if tick > 0 then
			if tick > d then
				tick = tick - d
			else
				tick = 0
			end
			self:seekWidgetByName('leb_time'):setText(getTickStr(tick))
		end
	end, 0)
	
	self:addLuaHandler('btn_release', function ()
		g_SendMsg({ptype = 'miner_rls', uid = miner.uid})
	end)
	self:addLuaHandler('btn_cancel', function () self:close() end)
	self:OutClickClose()
	
	self:regJMsgHandler('miner_rls', function () self:close() end)
	
	return true
end


----------------------------------------------------------------
----------------------------------------------------------------
-- 抓捕
DlgMinerList = class('DlgMinerList', LuaUI)
local super = DlgMinerList.__super

function DlgMinerList:init(msg, capCost)
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_fight.json', true) then return false end
	
	self:addLuaHandler('btn_change', function () g_SendMsg({ptype = 'miner_cap'}) end)
	self:seekWidgetByName('list_item'):setDirection(SCROLLVIEW_DIR_VERTICAL)
	self:OutClickClose()
	
	self.capCost = capCost
	self:regJMsgHandler('miner_cap', self.onRecvMinerList)
	self:onRecvMinerList(msg)
	
	return true
end

function DlgMinerList:onRecvMinerList(msg)
	if type(msg) ~= 'table' then
		self:close()
		return
	end
	
	local lv = self:seekWidgetByName('list_item')
	while lv:getItems():count() > 0 do
		lv:removeLastItem()
	end
	local widget = LOADWIDGET('stope_item.json')
	for k, v in ipairs(msg) do
		local w = widget:clone()
		local imgIcon = UIHO:seekWidgetByName(w, 'img_icon')
		imgIcon:setVisible(true)
		imgIcon:setScale(1.35)
		imgIcon:loadTexture(heroHead[v.race+1], UI_TEX_TYPE_PLIST)
		UIHO:seekWidgetByName(w, 'leb_name'):setText(SSFmt('%s(Lv.%d)', v.name, v.level))
		UIHO:seekWidgetByName(w, 'leb_identity'):setText(TDGETSTRCS('SD_QUARRY_STATUS_'..v.status))
		UIHO:seekWidgetByName(w, 'Label_57'):setVisible(false)
		UIHO:seekWidgetByName(w, 'leb_faction'):setText(v.faction or TDGETSTRCS('PLAYER_NO_FUNCTION'))
		local btnCapture = UIHO:seekWidgetByName(w, 'btn_rescue')
		btnCapture:loadTextures('btn_zb.png', 'btn_zb.png', 'btn_zb.png', UI_TEX_TYPE_PLIST)
		self:addLuaHandler(btnCapture, function ()
			if tonumber(v.uid) <= 0xffffffff then
				LShowMessage(SSFmt(TDGETSTRCS('SD_QUARRY_CAP_TIP'), self.capCost),
				nil, nil, MBT_OKCANCEL,
				function ()
					g_SendMsg({ptype='miner_cap', uid = v.uid})
				end)
			else
				g_SendMsg({ptype='miner_cap', uid = v.uid})
			end
		end)
		lv:pushBackCustomItem(w)
	end
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 营救
DlgRescueList = class('DlgRescueList', LuaUI)
local super = DlgRescueList.__super

function DlgRescueList:init(msg)
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_fight.json', true) then return false end
	
	self:seekWidgetByName('btn_change'):setEnabled(false)
	self:seekWidgetByName('list_item'):setDirection(SCROLLVIEW_DIR_VERTICAL)
	self:OutClickClose()
	
	self:regJMsgHandler('miner_rsc', self.onRecvMinerRescue)
	self:onRecvMinerRescue(msg)
	
	return true
end

function DlgRescueList:onRecvMinerRescue(msg)
	if type(msg) ~= 'table' then
		self:close()
		return
	end
	
	local lv = self:seekWidgetByName('list_item')
	while lv:getItems():count() > 0 do
		lv:removeLastItem()
	end
	local widget = LOADWIDGET('stope_item.json')
	for k, v in ipairs(msg) do
		local w = widget:clone()
		local imgIcon = UIHO:seekWidgetByName(w, 'img_icon')
		imgIcon:setVisible(true)
		imgIcon:setScale(1.35)
		imgIcon:loadTexture(heroHead[v.race+1], UI_TEX_TYPE_PLIST)
		UIHO:seekWidgetByName(w, 'leb_name'):setText(SSFmt('%s(Lv.%d)', v.name, v.level))
		UIHO:seekWidgetByName(w, 'leb_identity'):setText(TDGETSTRCS('SD_QUARRY_STATUS_'..v.status))
		UIHO:seekWidgetByName(w, 'Label_57'):setVisible(false)
		UIHO:seekWidgetByName(w, 'Label_48'):setVisible(false)
		UIHO:seekWidgetByName(w, 'leb_faction'):setVisible(false)
		local btnCapture = UIHO:seekWidgetByName(w, 'btn_rescue')
		btnCapture:loadTextures('btn_yj.png', 'btn_yj.png', 'btn_yj.png', UI_TEX_TYPE_PLIST)
		self:addLuaHandler(btnCapture, function () g_SendMsg({ptype='miner_rsc', uid = v.uid}) end)
		lv:pushBackCustomItem(w)
	end
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 复仇
DlgAvengeList = class('DlgAvengeList', LuaUI)
local super = DlgAvengeList.__super

function DlgAvengeList:init(msg)
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_fight.json', true) then return false end
	
	self:seekWidgetByName('btn_change'):setEnabled(false)
	self:seekWidgetByName('list_item'):setDirection(SCROLLVIEW_DIR_VERTICAL)
	self:OutClickClose()
	
	self:regJMsgHandler('miner_avg', self.onRecvMinerAvenge)
	self:onRecvMinerAvenge(msg)
	
	return true
end

function DlgAvengeList:onRecvMinerAvenge(msg)
	if type(msg) ~= 'table' then
		self:close()
		return
	end
	
	local lv = self:seekWidgetByName('list_item')
	while lv:getItems():count() > 0 do
		lv:removeLastItem()
	end
	local widget = LOADWIDGET('stope_item.json')
	for k = #msg, 1, -1 do
		local v = msg[k]
		local w = widget:clone()
		local imgIcon = UIHO:seekWidgetByName(w, 'img_icon')
		imgIcon:setVisible(true)
		imgIcon:setScale(1.35)
		imgIcon:loadTexture(heroHead[v.race+1], UI_TEX_TYPE_PLIST)
		UIHO:seekWidgetByName(w, 'leb_name'):setText(SSFmt('%s(Lv.%d)', v.name, v.level))
		UIHO:seekWidgetByName(w, 'leb_identity'):setText(TDGETSTRCS('SD_QUARRY_STATUS_'..v.status))
		UIHO:seekWidgetByName(w, 'Label_57'):setVisible(false)
		UIHO:seekWidgetByName(w, 'leb_faction'):setText(v.faction or TDGETSTRCS('PLAYER_NO_FUNCTION'))
		local btnCapture = UIHO:seekWidgetByName(w, 'btn_rescue')
		btnCapture:loadTextures('btn_gd.png', 'btn_gd.png', 'btn_gd.png', UI_TEX_TYPE_PLIST)
		self:addLuaHandler(btnCapture, function () g_SendMsg({ptype='miner_avg', uid = v.uid}) end)
		lv:pushBackCustomItem(w)
	end
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 求救
DlgSOSList = class('DlgSOSList', LuaUI)
local super = DlgSOSList.__super

function DlgSOSList:init(msg)
	if not super.init(self) then return false end
	if not self:initRootWidget('stope_fight.json', true) then return false end
	
	self:seekWidgetByName('btn_change'):setEnabled(false)
	self:seekWidgetByName('list_item'):setDirection(SCROLLVIEW_DIR_VERTICAL)
	self:OutClickClose()
	
	self:regJMsgHandler('miner_sos', self.onRecvMinerSOS)
	self:onRecvMinerSOS(msg)
	
	return true
end

function DlgSOSList:onRecvMinerSOS(msg)
	if type(msg) ~= 'table' then
		self:close()
		return
	end
	
	local lv = self:seekWidgetByName('list_item')
	while lv:getItems():count() > 0 do
		lv:removeLastItem()
	end
	local widget = LOADWIDGET('stope_item.json')
	for k, v in ipairs(msg) do
		local w = widget:clone()
		local imgIcon = UIHO:seekWidgetByName(w, 'img_icon')
		imgIcon:setVisible(true)
		imgIcon:setScale(1.35)
		imgIcon:loadTexture(heroHead[v.race+1], UI_TEX_TYPE_PLIST)
		UIHO:seekWidgetByName(w, 'leb_name'):setText(SSFmt('%s(Lv.%d)', v.name, v.level))
		UIHO:seekWidgetByName(w, 'leb_identity'):setText(TDGETSTRCS('SD_QUARRY_STATUS_'..v.status))
		UIHO:seekWidgetByName(w, 'leb_times'):setText(v.cnt)
		UIHO:seekWidgetByName(w, 'Label_48'):setVisible(false)
		UIHO:seekWidgetByName(w, 'leb_faction'):setVisible(false)
		local btnCapture = UIHO:seekWidgetByName(w, 'btn_rescue')
		btnCapture:loadTextures('btn_qj_01.png', 'btn_qj_01.png', 'btn_qj_01.png', UI_TEX_TYPE_PLIST)
		self:addLuaHandler(btnCapture, function () g_SendMsg({ptype='miner_sos', uid = v.uid}) end)
		lv:pushBackCustomItem(w)
	end
end