local json = require'json'
require'uifunc'

local SSFmt = string.format

RedActivity = class('RedActivity', LuaUI)
local super = RedActivity.__super

local g_cutDown = nil
local g_rob = nil

local function getTime(tm)
	local d = math.floor(tm / 86400)
	tm = tm % 86400
	local h = math.floor(tm / 3600)
	tm = tm % 3600
	local m = math.floor(tm / 60)
	tm = tm % 60
	return d, h, m, tm
end

function RedActivity:init()
	if not super.init(self) then return false end
	
	if not self:initRootWidget('activity_red.json', true) then return false end
	
	self:OutClickClose()
	self:addLuaHandler('btn_rob', function () g_SendMsg({ptype = 'redactaward'}) end)
	
	g_SendMsg({ptype = 'redactopen'})
	
	return true
end

function RedActivity:onEnter()
	self:regJMsgHandler('redact', self.ActivityInfo)
	self:regJMsgHandler('redcutdown', self.UpdateUI)
end

function RedActivity:ActivityInfo(data)
	--活动倒计时
	local bt = data.bt
	local et = data.et
	local nt = data.nt
	self:updateTime(bt, et, nt)
	schedule(self, function ()
		nt = nt + 1
		self:updateTime(bt, et, nt)
	end, 1)
end

function RedActivity:updateTime(bt, et, nt)
	local txt = self:seekWidgetByName('lab_endTime')
	local tip1
	if nt < bt then
		tip1 = string.format(getString('SD_ACTIVITY_BEGIN'),
			getTime(bt - nt))
	elseif nt > et then
		tip1 = (getString('SD_ACTIVITY_END'))
	else
		tip1 = string.format(getString('SD_ACTIVITY_ING'),
			getTime(et - nt))
	end
	txt:setText(string.format('%s\n', tip1))
end

function RedActivity:UpdateUI(data)
	local et = data.next
	local nt = data.nt
	local rob = data.rob
	local times = data.times
	local gived = data.gived
	
	local txt = self:seekWidgetByName('Label')
	if et == 0 and times ~= 0 then
		txt:setText(SSFmt(getString('RED_TIPS_1'), times))
	else
		if times == 0 then
			txt:setText(SSFmt(getString('RED_TIPS_3'), times))
		else
			txt:setText(SSFmt(getString('RED_TIPS_2'), times))
		end
	end
	
	local btn_rob = self:seekWidgetByName("btn_rob")
	if rob == 1 and gived == 0 then
		local actions = CCArray:create()
		actions:addObject(CPScaleTo:create(0.9,0.9))
		actions:addObject(CPScaleTo:create(0.9,1.2))
		btn_rob:runAction(CPRepeatForever:create(CPSequence:create(actions)))
		btn_rob:setTouchEnabled(true)
		btn_rob:setVisible(true)
	else
		btn_rob:setTouchEnabled(false)
		btn_rob:setVisible(false)
	end
	
	local cutdownInfo = self:seekWidgetByName('Image_cutdown')
	if cutdownInfo ~= nil then
		cutdownInfo:loadTexture(rob == 1 and 'icon_txt_redbag3.png' or 'icon_txt_redbag4.png', UI_TEX_TYPE_PLIST)
	end
	
	local cutdownSec = self:seekWidgetByName('Image_Sec')
	if cutdownSec ~= nil then
		cutdownSec:loadTexture(rob == 1 and 'icon_txt_redbag6.png' or 'icon_txt_redbag5.png', UI_TEX_TYPE_PLIST)
	end
	
	if g_cutDown ~= nil then 
		self:stopAction(g_cutDown)
		g_cutDown = nil
	end
	
	self:updateTimeEx(et, nt)
	g_cutDown = schedule(self, function ()
		nt = nt + 1
		self:updateTimeEx(et, nt)
	end, 1)
end

function RedActivity:updateTimeEx(et, nt)
	local txt = self:seekWidgetByName('bit_time')
	if nt < et then
		txt:setText(SSFmt("%u", et - nt))
	else
		txt:setText(SSFmt("%u", 0))
	end
end

function RedActivity:_nil()
end