local bit = require'bit'
local itemCfg = require'config.item'

local tcast = tolua.cast

SignActivity = class('SignActivity', LuaUI)
local super = SignActivity.__super

function SignActivity:init()
	if not super.init(self) then return false end
	
	if not self:initRootWidget('sign_in.json', true) then
		return false
	end
	
	self:addLuaHandler('btn_close', function () self:close() end)
	self:addLuaHandler('btn_receive', function () g_SendMsg({ptype = 'sign_award'}) end)
	self:seekWidgetByName('leb_month'):setText(SSFmt(TDGETSTRCS('SIGN_ACTIVITY_DESC'), os.date('*t').month))
	
	self:regJMsgHandler('sign_list', self.onRecvSignList)
	self:regJMsgHandler('sign_flag', self.onRecvSignFlag)
	g_Player:GetSignList()
	g_SendMsg({ptype = 'sign_flag'})
	
	if g_Player.signList then
		self:updateSignList(g_Player.signList)
	end
	
	return true
end

function SignActivity:onRecvSignList(msg)
	self:updateSignList(msg.list)
end

function SignActivity:onRecvSignFlag(flag)
	self.signFlag = flag
	self:updateSignFlag(flag)
end

local maxDay = {31,28,31,30,31,30,31,31,30,31,30,31}
local function isLeapYear(year)
	return (year%4==0 and year%100~=0) or (year%400==0)
end
local function getMaxDay()
	local dt = os.date('*t')
	if dt.month == 2 and isLeapYear(dt.year) then
		return 29
	end
	return maxDay[dt.month]
end
function SignActivity:updateSignList(list)
	local lv = self:seekWidgetByName('scv_list')
	local children = lv:getChildren()
	while children:count() > 0 do
		lv:removeLastItem()
	end
	
	local p
	local widget = LOADWIDGET('sign_in_item.json')
	local maxDay = getMaxDay()
	for k, v in ipairs(list) do
		if v.id > maxDay then break end
		local idx = (k - 1) % 3
		if idx == 0 then
			p = Layout:create()
			p:setSize(CCSizeMake(552, 180))
			lv:pushBackCustomItem(p)
		end
		local w = widget:clone()
		
		if v.money then  -- 
			UIHO:seekWidgetByName(w, 'itemImg'):loadTexture('img_bg_money.png', UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(w, 'lab_dec'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_MONEY'), v.money))
		elseif v.crystal then  -- ʯ
			UIHO:seekWidgetByName(w, 'itemImg'):loadTexture('img_bg_051.png', UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(w, 'lab_dec'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_CRYSTAL'), v.crystal))
		elseif v.itemId and v.itemCnt then  -- 
			local cfg = itemCfg[v.itemId]
			UIHO:seekWidgetByName(w, 'itemImg'):loadTexture(cfg.texture, UI_TEX_TYPE_PLIST)
			UIHO:seekWidgetByName(w, 'lab_dec'):setText(cfg.name..'*'..v.itemCnt)
		end
		-- VIPǷ񷭱
		if v.vip then
			UIHO:seekWidgetByName(w, 'Image_115'):setVisible(true)
			UIHO:seekWidgetByName(w, 'vip_txt'):setText(tostring(v.vip))
		else
			UIHO:seekWidgetByName(w, 'Image_115'):setVisible(false)
		end
		UIHO:seekWidgetByName(w, 'leb_day'):setText(SSFmt(TDGETSTRCS('DAILY_TASK_DAY'), v.id))
		
		w:setPosition(ccp(idx * 184 + 9, 8))
		w:setName('item_'..v.id)
		p:addChild(w)
	end
	if self.signFlag then
		self:updateSignFlag(self.signFlag)
	end
end

function SignActivity:updateSignFlag(flag)
	local lv = self:seekWidgetByName('scv_list')
	for k = 1, 31 do
		local n = UIHO:seekWidgetByName(lv, 'item_'..k)
		if n then
			if bit.test(flag, k) then
				UIHO:seekWidgetByName(n, 'Image_116'):setVisible(true)
			else
				UIHO:seekWidgetByName(n, 'Image_116'):setVisible(false)
			end
		end
	end
end