local itemCfg = require'config.item'
local starCfg = require'config.star'
local gemCfg = require'config.gem'
local gemEquipCfg = require'config.gemequip'

local SSFmt = string.format

local tcast = tolua.cast

local MakeItemDetail = {}
local MakeItemDetail = class('MakeItemDetail')
local super = MakeItemDetail.__super

local raceName = { 
	"SD_Warrior", 
	"SD_Mage", 
	"SD_Priest",
}

local colors = {
	{ 255, 255, 255 },
	{ 140, 255, 130 },
	{ 100, 150, 255 },
	{ 255, 0, 255 },
	{ 255, 127, 0 },
	{ 255, 127, 0 },
}

local equipStarNameConvertor = {
	"DAMAGE",
	"CRITICAL",
}

local specialBaseProperty = {
	2.0,	-- EQUIPMENT_TYPE_MAIN_WEAPON = 0,
	3.0,	-- EQUIPMENT_TYPE_SUB_WEAPON = 1,
	0.1,	-- EQUIPMENT_TYPE_HEAD = 2,
	1.5,	-- EQUIPMENT_TYPE_BODY = 3,
	2.0,	-- EQUIPMENT_TYPE_LEG = 4,
	5.0,	-- EQUIPMENT_TYPE_GLOVE = 5,
	5.0,	-- EQUIPMENT_TYPE_RING = 6,
	10.0,	-- EQUIPMENT_TYPE_NECKLACE = 7,
	3.0,	-- EQUIPMENT_TYPE_WAIST = 8,
	5.0,	-- EQUIPMENT_TYPE_BOOTS = 9,
}

local gemName = {
	"Ruby",
	"Emerald",
	"Sapphire",
	"Topaz",
}

local gemEffect = {
	"SD_EQUIP_STR",	-- 力量
	"SD_EQUIP_DEX",	-- 敏捷
	"SD_EQUIP_INT",	-- 治疗
	"SD_EQUIP_STA",	-- 耐力
}

local equipNameConvertor = {
	"MAIN_WEAPON",	-- EQUIPMENT_TYPE_MAIN_WEAPON = 0,
	"SUB_WEAPON",	-- EQUIPMENT_TYPE_SUB_WEAPON = 1,
	"HEAD",			-- EQUIPMENT_TYPE_HEAD = 2,
	"BODY",			-- EQUIPMENT_TYPE_BODY = 3,
	"LEG",			-- EQUIPMENT_TYPE_LEG = 4,
	"GLOVE",		-- EQUIPMENT_TYPE_GLOVE = 5,
	"RING",			-- EQUIPMENT_TYPE_RING = 6,
	"NECKLACE",		-- EQUIPMENT_TYPE_NECKLACE = 7,
	"WAIST",		-- EQUIPMENT_TYPE_WAIST = 8,
	"BOOTS",		-- EQUIPMENT_TYPE_BOOTS = 9,
}

local equipIdConvertor = {
	"icon_txt2_zswq.png", 	-- EQUIPMENT_TYPE_MAIN_WEAPON = 0,
	"icon_txt2_fswq.png", 	-- EQUIPMENT_TYPE_SUB_WEAPON = 1,
	"icon_txt2_tk.png", 	-- EQUIPMENT_TYPE_HEAD = 2,
	"icon_txt2_xj.png", 	-- EQUIPMENT_TYPE_BODY = 3,
	"icon_txt2_ht.png", 	-- EQUIPMENT_TYPE_LEG = 4,
	"icon_txt2_st.png", 	-- EQUIPMENT_TYPE_GLOVE = 5,
	"icon_txt2_jz.png", 	-- EQUIPMENT_TYPE_RING = 6,
	"icon_txt2_xl.png", 	-- EQUIPMENT_TYPE_NECKLACE = 7,
	"icon_txt2_yd.png", 	-- EQUIPMENT_TYPE_WAIST = 8,
	"icon_txt2_xz.png", 	-- EQUIPMENT_TYPE_BOOTS = 9,
}

function MakeItemDetail:ChangelScore(iiv, uid, bOther)
	local item = g_Player:GetItem(uid)
	if item ~= nil and iiv ~= nil then
		local equipBaseScore, equipScoreadd = item:GetScore()
		local children = iiv:getChildren()
		if children ~= nil then 
			for k = 1, children:count() do
				local w = tcast(children:objectAtIndex(k), 'UIStaticRichText')
				local info = w:getStringValue()
				if info ~= nil then
					local infoEx = nil
					local infoEx1 = nil
					if equipScoreadd > 1 then 
						local pat = string.format('<c=ff00ff><s=20>%s%u</></>', getString('SD_EQUIP_SCORE'), equipBaseScore + equipScoreadd)
						local repat = string.format('<c=ff00ff><s=20>%s%u</></><s=20><c=ff7D00>+%u</></>', getString('SD_EQUIP_SCORE'), equipBaseScore, equipScoreadd)
						infoEx = string.gsub(info, pat, repat)
					else
						infoEx = info
					end
					
					if bOther then 
						infoEx1 = string.gsub(infoEx, '::0.56', '::0.43')
					else
						infoEx1 = infoEx
					end
					w:setText(infoEx1)
				end
			end
		end
	end
end

function MakeItemDetail:UpdateBaseProperty(_name, value, enhancedLevel, gemApp)
	local equipInfo = nil
	if value > 0 then
		equipInfo = SSFmt('%s %u\n', TDGETSTRCS('SD_EQUIP_'.._name), enhancedLevel * value + gemApp * value /100)
	end
	return equipInfo
end

function MakeItemDetail:addSqAnim(widget, aniname, tag, sc, frameNum)
	local effectSpr = CreateAnimation(aniname, 'main-equip.plist', 0.2, frameNum)
	if effectSpr ~= nil then
		effectSpr:setScale(sc)
		effectSpr:setOpacity(200)
		widget:addNode(effectSpr, 10, tag)
	end
end

function MakeItemDetail:addStarAnim(widget, aniname, tag, sc, frameNum)
	local effectSpr = CreateAnimation(aniname, 'main_effects.plist', 0.2, frameNum)
	if effectSpr ~= nil then
		effectSpr:setScale(sc)
		effectSpr:setOpacity(200)
		widget:addNode(effectSpr, 10, tag)
	end
end

function MakeItemDetail:getGemEquipCfg(gemLv)
	return gemEquipCfg[math.floor(gemLv / 4) * 4]
end

function MakeItemDetail:getNextGemEquipCfg(gemLv)
	return gemEquipCfg[math.ceil(gemLv / 4) * 4]
end

function MakeItemDetail:ItemDetail(equip, infoPanel, itemType, itemBoarder, packMode, tipsType, offser, needChangeSize)
	if equip == nil then return end

	local ids = itemCfg[equip.templateId]
	if ids == nil then return end

	local ssc = starCfg[equip.m_starNum]
	local addRatio = 1.0
	if ssc ~= nil and tipsType ~= 9 then addRatio = addRatio + ssc.base_attr / 10000.0 end

	if itemBoarder ~= nil then
		local colorex = colors[equip.m_randomPropertyCount+1]
		itemBoarder:setColor(ccc3(colorex[1], colorex[2], colorex[3]))
		local itemIcon = itemBoarder:getChildByName("item")
		if itemIcon ~= nil then
			itemIcon:setVisible(true)
			local node = itemIcon:getNodeByTag(-100)
			while node ~= nil do
				itemIcon:removeNode(node) 
				node = itemIcon:getNodeByTag(-100)
			end
			
			local spr0 = CPSprite:createWithSpriteFrameName('equip_frame_01.png')
			-- 兼容传奇挂机
			if not spr0:getContentSize():equals(CCSize(104, 104)) then 
				spr0 = CPSprite:createWithSpriteFrameName('img_bg_011.png')
			end
			local spr = CPSprite:createWithSpriteFrameName(ids.texture)
			if spr ~= nil then
				spr0:setColor(ccc3(colorex[1], colorex[2], colorex[3]))
				spr:setPosition(ccp(spr0:getContentSize().width/2, spr0:getContentSize().height/2))
				spr0:addChild(spr)
				itemIcon:addNode(spr0, 1, -100)
				itemIcon:setSize(CCSizeMake(spr:getContentSize().width, spr:getContentSize().height))
			end
			
			if equip.m_starNum > 5 then
				self:addStarAnim(itemIcon, equip.m_starNum >= 18 and 'sun%u.png' or 'moon%u.png', -100, 2.0, 8)
			end
			
			if equip:getSqType() == 1 then 
				self:addSqAnim(itemIcon, 'sq_%03u.png', -100, 1.0, 14) 
			elseif equip:getSqType() == 2 then 
				self:addSqAnim(itemIcon, 'k_%03u.png', -100, 1.2, 15)
			elseif equip:getSqType() == 4 then 
				self:addSqAnim(itemIcon, 'sq_%03u.png', -100, 1.0, 14)
				self:addSqAnim(itemIcon, 'k_%03u.png', -100, 1.2, 15)
			end
		end
	end

	if infoPanel == nil then return end

	local item = g_Player:GetItem(equip.uniqueId)
	local equipEhancedLevel = equip.m_enhancedLevel
	local equipInfo
	if equipEhancedLevel ~= 0 then
		equipInfo = SSFmt('<c=ffcc88><s=30>Lv%u %s+%u</></>\n', ids.level, item:getColorNameEx(), equipEhancedLevel)
	else
		equipInfo = SSFmt('<c=ffcc88><s=30>Lv%u %s</></>\n', ids.level, item:getColorNameEx());
	end
	
	if ids.requireRace ~= 3 then
		local fmt = SSFmt(TDGETSTRCS('SD_RACE_REQUER1'), TDGETSTRCS(raceName[ids.requireRace%3 + 1]))
		equipInfo = equipInfo..SSFmt('<c=%s><s=20>%s</></>\n', (PLAYER_BASE_DATA.race ~= ids.requireRace) and 'ff4444' or '44ff44', fmt)
	end

	local equipBaseScore, equipScoreadd = item:GetScore()
	if equipScoreadd > 1 then
		equipInfo = equipInfo..SSFmt('<c=ff00ff><s=20>%s%u</></><s=20><c=ff7D00>+%u</></>\n', TDGETSTRCS('SD_EQUIP_SCORE'), equipBaseScore, equipScoreadd)
	else
		equipInfo = equipInfo..SSFmt('<c=ff00ff><s=20>%s%u</></>\n', TDGETSTRCS('SD_EQUIP_SCORE'), equipBaseScore)
	end

    -- 强化属性只提升固定属性部分
    -- 提升规则为 elevel * 2% * color
	local enhancedLevel = 1 + equipEhancedLevel * 0.02 * equip.m_randomPropertyCount
	enhancedLevel = addRatio * enhancedLevel

	local gemLv = equip:getGemLv()
	local gemApp = 0
	local sgc = self:getGemEquipCfg(gemLv)
	if sgc ~= nil then gemApp = sgc.ratio end
	if ids.damageMin ~= 0 or ids.damageMax ~= 0 then
		equipInfo = equipInfo..SSFmt('<s=30>%s%u - %u</>\n', TDGETSTRCS('SD_DAMAGE'), 
			enhancedLevel*ids.damageMin + ids.damageMin*gemApp / 100, enhancedLevel*ids.damageMax + ids.damageMax*gemApp / 100)
	end
	
	if ids.armor > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('armor', ids.armor, enhancedLevel, gemApp) end
	if ids.accuracy > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('accuracy', ids.accuracy, enhancedLevel, gemApp) end
	if ids.dodge > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('dodge', ids.dodge, enhancedLevel, gemApp) end
	if ids.critical > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('critical', ids.critical, enhancedLevel, gemApp) end
	if ids.hp > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('hp', ids.hp, enhancedLevel, gemApp) end
	if ids.mp > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('mp', ids.mp, enhancedLevel, gemApp) end
	if ids.decCritical > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('decCritical', ids.decCritical, enhancedLevel, gemApp) end
	if ids.physicalResistance > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('physicalResistance', ids.physicalResistance, enhancedLevel, gemApp) end
	if ids.magicResistance > 0 then equipInfo = equipInfo..self:UpdateBaseProperty('magicResistance', ids.magicResistance, enhancedLevel, gemApp) end
	
	if ssc ~= nil then
		local sun = math.floor(equip.m_starNum / 18)
		local moon = math.floor(equip.m_starNum % 18 / 6)
		local star = equip.m_starNum % 6
		
		for i = 1, sun do
			equipInfo = equipInfo..'<img=2M_icon_img_hd:sun.png>'
		end
		
		for i = 1, moon do
			equipInfo = equipInfo..'<img=2M_icon_img_hd:moon.png>'
		end
		
		for i = 1, star do
			equipInfo = equipInfo..'<img=2M_icon_img_hd:star.png>'
		end
		
		if tipsType == 9 then
			equipInfo = equipInfo..SSFmt('<s=20>%s</>\n', SSFmt(TDGETSTRCS('ST_STAR_ATTR_UNKNOWN')))
		else
			equipInfo = equipInfo..SSFmt('<s=20>%s</>', SSFmt(TDGETSTRCS('ST_STAR_ATTR_NAME'), 
			buildStarAttrDesc(equip.m_starAttrType, equip.m_starAttrValue)))
		end
	end
	
	if tipsType == 9 then
		equipInfo = equipInfo..SSFmt(TDGETSTRCS('SD_EquipBuyInfo'), equip.m_randomPropertyCount)
	else
		for k = 1, 4, 1 do 
			local v = equip:GetRandmProperty(k)
			if v ~= 0 then 
				equipInfo = equipInfo..SSFmt('<c=ffdd00><s=20>%s +%u</></>\n', TDGETSTRCS(gemEffect[k]), v)
			end
		end
	end

	if packMode < 1 then
		if equip.m_specialProperty ~= 0 then
			local _exp = 0
			local level = equip:GetSpecialLevel(_exp)
			local detail = SSFmt(TDGETSTRCS('ST_SPECIAL_NAME'), level, _exp)
			equipInfo = equipInfo..SSFmt('<c=ff0000><s=20>%s</></>\n', detail)
			equipInfo = equipInfo..SSFmt('<c=ff0000><s=20>%s</></>\n', buildSqAttrDesc(nil, ids.equipType, level))

			if tipsType ~= 8 and tipsType ~= 9 then
				local fullEnhanceLevel = MAIN_PLAYER.getFullEnhancedLevel()
				if fullEnhanceLevel > level then fullEnhanceLevel = level end
				if fullEnhanceLevel ~= 0 then
					equipInfo = equipInfo..SSFmt('<c=88ff88><s=20>%s%s</></>\n', 
						buildSqAttrDesc(nil, ids.equipType, fullEnhanceLevel), TDGETSTRCS('SQ_ACTIVATED'))
				end
				if fullEnhanceLevel < MAIN_PLAYER.sqexp.size()- 1 then
					if level >= fullEnhanceLevel + 1 then
						equipInfo = equipInfo..SSFmt('<c=888888><s=20>%s%s</></>\n', buildSqAttrDesc(nil, ids.equipType, fullEnhanceLevel+1), 
							SSFmt(TDGETSTRCS('SQ_NOT_ACTIVATED_0'), fullEnhanceLevel+1))
					else 
						equipInfo = equipInfo..SSFmt('<c=888888><s=20>%s%s</></>\n', buildSqAttrDesc(nil, ids.equipType, level+1), 
							SSFmt(TDGETSTRCS('SQ_NOT_ACTIVATED_1'), level+1))
					end
				end
			end
		end
		-- 第二神器
		if equip.m_2specialProperty ~= 0 then
			local _exp = 0
			local level = equip:Get2SpecialLevel(_exp)
			local detail = SSFmt(TDGETSTRCS('ST_SPECIAL2_NAME'), level, _exp)
			equipInfo = equipInfo..SSFmt('<c=ff0000><s=20>%s</></>\n', detail)
			equipInfo = equipInfo..SSFmt('<c=ff0000><s=20>%s</></>\n', buildSq2AttrDesc(ids.equipType, level*MAIN_PLAYER.sq2evalue[ids.equipType]))
			
			if level < MAIN_PLAYER.sq2exp.size() - 1 then
				level = level + 1
				equipInfo = equipInfo..SSFmt('<c=888888><s=20>%s%s</></>\n', 
					buildSq2AttrDesc(ids.equipType, level*MAIN_PLAYER.sq2evalue[ids.equipType]), SSFmt(TDGETSTRCS("SQ_NOT_ACTIVATED_2"), level))
			end
		end
		
		-- 宝石套装
		local gemsuit
		if sgc ~= nil then
			gemsuit = gemsuit..SSFmt(TDGETSTRCS('SD_GEM_EQUIP'), sgc.tip)
			gemsuit = gemsuit..'\n'
		end

		local sgc = self:getNextGemEquipCfg(gemLv+1)
		if sgc ~= nil then
			gemsuit = gemsuit..SSFmt(TDGETSTRCS('SD_GEM_EQUIPNEXT'), sgc.tip, sgc.level)
		end

		gemsuit = SSFmt('<s=20>%s\n</>', gemsuit)

		equipInfo = equipInfo..gemsuit
	end

	if packMode < 2 then
		for  i = 1, 4, 1 do
			local gemTemplateId = equip.m_jewelProperty[i]
			local config = itemCfg[gemTemplateId]
			local gemCfg = gemCfg[gemTemplateId]
			if gemCfg ~= nil and config ~= nil then
				local abilityTip
				if gemCfg.power > 0 then abilityTip = abilityTip..SSFmt('%s+%u ', TDGETSTRCS(gemEffect[1]), gemCfg.power) end
				if gemCfg.agile > 0 then abilityTip = abilityTip..SSFmt('%s+%u ', TDGETSTRCS(gemEffect[2]), gemCfg.agile) end
				if gemCfg.intelligence > 0 then abilityTip = abilityTip..SSFmt('%s+%u ', TDGETSTRCS(gemEffect[3]), gemCfg.intelligence) end
				if gemCfg.endurance > 0 then abilityTip = abilityTip..SSFmt('%s+%u', TDGETSTRCS(gemEffect[4]), gemCfg.endurance) end

				equipInfo = equipInfo..SSFmt('<c=ff00ff><s=20>%s %s</></>\n', config.name, abilityTip)
			end
		end
	end

	if packMode < 3 then
		for i = 1, 4, 1 do
			if equip.m_jewelProperty[i] ~= 0 then
				if equip.m_jewelProperty[i] == 0xffffffff then
					equipInfo = equipInfo..SSFmt('<img=btn_34.png> ')
				else
					local gemTemplateId = equip.m_jewelProperty[i]
					local idss = itemCfg[gemTemplateId]
					if idss ~= nil then
						equipInfo = equipInfo..SSFmt('<img=btn_34.png,item:%s::0.43> ', idss.texture)
					end
				end
			end
		end
	end

	infoPanel:removeAllChildren()

	local srt = UIStaticRichText:createWithString(equipInfo, infoPanel:getSize().width + 10, 5)
	srt:setPosition(ccp(4 + srt:getSize().width / 2, infoPanel:getSize().height - srt:getSize().height / 2 + offser))
	infoPanel:addChild(srt)

	if needChangeSize == true then
		local infoSize = srt:getSize()
		if infoSize.height + 8 > infoPanel:getSize().height then
			infoPanel:setSize(CCSizeMake(infoPanel:getSize().width, infoSize.height))
		end
	end
end

return MakeItemDetail
