local type = type
local assert = assert
local pcall = pcall
local loadstring = loadstring

local json = require'json'

--////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////
--// LUA消息处理
local allHandlers = {}
function regMsgHandler(proto, obj, func)
	assert(type(proto) == 'string', 'regMsgHandler: error protocol type')
	assert(type(func) == 'function', 'regMsgHandler: handler must be a function')
	allHandlers[proto] = allHandlers[proto] or {}
	allHandlers[proto][obj] = func
end
function unregAllMsgHandler(obj)
	for _, handlers in pairs(allHandlers) do
		handlers[obj] = nil
	end
end

----------------------------------------------------------------
--// C++收到LUA消息后调用该函数进行处理
--// 消息格式为 {proto='消息协议',d1=xxx,d2=xxx,...}
--// proto为消息协议，其余字段为消息数据
function onLuaMsgComing(str)
	local f = loadstring('return {'..str..'}')
	if not f then
		print('Can`t parse message: '..str)
		return
	end
	local r, msg = pcall(f)
	if not r then
		print('Can`t parse message: '..str)
		return
	end
	
	local handlers = allHandlers[msg.proto]
	if not handlers then return end
	
	for o, f in pairs(handlers) do
		local r, info = pcall(f, o, msg)
		if not r then print(info) end
	end
end



--////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////
--// JSON消息处理
local allJHandlers = {}
function regJMsgHandler(proto, obj, func)
	assert(type(proto) == 'string', 'regJMsgHandler: error protocol type')
	assert(type(func) == 'function', 'regJMsgHandler: handler must be a function')
	allJHandlers[proto] = allJHandlers[proto] or {}
	allJHandlers[proto][obj] = func
end
function unregAllJMsgHandler(obj)
	for _, handlers in pairs(allJHandlers) do
		handlers[obj] = nil
	end
end

----------------------------------------------------------------
--// C++收到JSON消息后调用该函数进行处理
--// 消息格式与老的消息格式保持一致
--// {"ptype":"消息协议","data":"消息数据"}
function onJsonMsgComing(str)
	local r, msg = pcall(json.decode, str)
	if not r then
		print('Can`t parse message: '..str..' Err: '..msg)
		return
	end
	
	local ptype = msg.ptype
	local data = msg.data
	if ptype == nil or data == nil then
		print('Json message format error, no `ptype` or `data`')
		return
	end
	
	local handlers = allJHandlers[ptype]
	if not handlers then return end
	
	local needSkip = false
	for o, f in pairs(handlers) do
		local r, info = pcall(f, o, data)
		if not r then
			print(info)
		elseif info == false then
			needSkip = true
			break
		end
	end
	return needSkip
end



--////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////
--// C++消息处理
local allCHandlers = {}
function regCMsgHandler(proto, obj, func)
	assert(type(proto) == 'number', 'regCMsgHandler: error protocol type')
	assert(type(func) == 'function', 'regCMsgHandler: handler must be a function')
	allCHandlers[proto] = allCHandlers[proto] or {}
	allCHandlers[proto][obj] = func
end
function unregAllCMsgHandler(obj)
	for _, handlers in pairs(allCHandlers) do
		handlers[obj] = nil
	end
end

----------------------------------------------------------------
--// C++收到消息后调用该函数进行处理
--// 消息内容为二进制流
--// 需要将该消息流解析为table后分发处理
local protocol = require'protocol'
local function tempty(t)
	for k, v in pairs(t) do
		return false
	end
	return true
end
function onMsgComing(proto, data, len, off)
	local handlers = allCHandlers[proto]
	if not handlers or tempty(handlers) then return end
	
	local name = protocol.ProtoNameMap_S2C_GAME_SERVER[proto]
	if not name then print('unknown protocol '..proto) end
	
	local cls = protocol[name]
	if not cls then print('unknown protocol '..name) end
	
	local msg = cls.new()
	msg:Unpack(data, len, off)
	
	local needSkip = false
	for o, f in pairs(handlers) do
		local r, info = pcall(f, o, msg)
		if not r then
			print(info)
		elseif info == false then
			needSkip = true
			break
		end
	end
	return needSkip
end