local unpack = unpack
local proto = require'protocol'
local mercCfg = require'config.mercenary'
local guideCfg = require'config.guide'
local guideMgr = require'guide'
require'fight'

IUEVENT_ADD = 0
IUEVENT_REMOVE = 1
IUEVENT_UPDATE = 2

Player = Player or class('Player')

function Player:ctor()
	self.abilityData = {}
	self.baseData = {}
	self.backpack = {}
	self.inventory = {}
	self.warehouse = {}
	self.mercenaryEuqip = {}
	self.mercenaries = {}
	self.betterEquip = {}
	self.betterEquipMerc = {}
	self.itemSyncHandler = {}
	self.propSyncHandler = {}
	self.syncFlags = 0
	self.buildEquipCost = 500
	self.fightTime = 0
	self.fightMsgs = {}
	self.fightMgr = FightMgr.new()
	self.fightMgr:SetFightType(FIGHT_TYPE_Normal)
	self.batFightMsgs = {}
	self.batFightMgr = FightMgr.new()
	self.batFightMgr:SetFightType(FIGHT_TYPE_BattleField)
	self.eliteFightMsgs = {}
	self.eliteFightMgr = FightMgr.new()
	self.eliteFightMgr:SetFightType(FIGHT_TYPE_EliteMap)
	self.quarryFightMsgs = {}
	self.quarryFightMgr = FightMgr.new()
	self.quarryFightMgr:SetFightType(FIGHT_TYPE_Quarry)
	self.abyssFightMsgs = {}
	self.abyssFightMgr = FightMgr.new()
	self.abyssFightMgr:SetFightType(FIGHT_TYPE_Abyss)
	local scheduler = CCDirector:sharedDirector():getScheduler()
	if self.updateID then
		scheduler:unscheduleScriptEntry(self.updateID)
	end
	self.updateID = scheduler:scheduleScriptFunc(
		function (d) self:update(d) end, 0, false)
		
	self.runeinventory = {}
	self.runebackpack = {}
	self.runeinventory_pos = {}
	self.runeitemSyncHandler = {}
	self.runebackpackSort ={}
	self.runeskill ={}
end

function Player:regRuneItemSyncHandler(obj, func)
	self.runeitemSyncHandler[obj] = func
end

function Player:unregRuneItemSyncHandler(obj)
	self.runeitemSyncHandler[obj] = nil
end

function Player:regItemSyncHandler(obj, func)
	self.itemSyncHandler[obj] = func
end
function Player:unregItemSyncHandler(obj)
	self.itemSyncHandler[obj] = nil
end
function Player:regPropSyncHandler(obj, func)
	self.propSyncHandler[obj] = func
end
function Player:unregPropSyncHandler(obj)
	self.propSyncHandler[obj] = nil
end

function Player:update(d)
	self.fightTime = self.fightTime + d
	if self.fightTime > 1 then
		self.fightTime = self.fightTime - 1
		self:ShowFightMsg()
	end
end

function Player:ShowFightMsg()
	local temp = {
		{self.fightMgr, self.fightMsgs},
		{self.batFightMgr, self.batFightMsgs},
		{self.eliteFightMgr, self.eliteFightMsgs},
		{self.quarryFightMgr, self.quarryFightMsgs},
		{self.abyssFightMgr, self.abyssFightMsgs},
	}
	for k, v in ipairs(temp) do
		local fightMgr, fightMsgs = v[1], v[2]
		if not fightMgr:Valid() and #fightMsgs > 0 then
			local msg = table.remove(fightMsgs, 1)
			fightMgr:SetMagData(msg)
			fightMgr:FightBegin()
		else
			fightMgr:ShowMsg()
		end
	end
end

function Player:onRecvSyncFightData(msg)
	if msg.fightResult.result < 100 then
		table.insert(self.fightMsgs, msg)
	elseif msg.fightResult.result < 200 then
		table.insert(self.batFightMsgs, msg)
	elseif msg.fightResult.result < 300 then
		table.insert(self.eliteFightMsgs, msg)
	elseif msg.fightResult.result < 400 then
		table.insert(self.quarryFightMsgs, msg)
		local ui = GetLuaUI('DlgQuarry')
		if ui ~= nil then
			DlgQuarryFight:Show(ui:GetRootLayer())
		end
	elseif msg.fightResult.result < 500 then
		table.insert(self.abyssFightMsgs, msg)
		-- local ui = GetLuaUI('AbyssChallenge')
		-- if ui ~= nil then
			-- DlgAbyssFight:Show(ui:GetRootLayer())
		-- end
	end
	return false
end

function Player:onRecvInitFightData(msg)
	local ret = true
	local fightMgr
	if msg.fightResult.result < 100 then
		fightMgr = self.fightMgr
	elseif msg.fightResult.result < 200 then
		fightMgr = self.batFightMgr
	elseif msg.fightResult.result < 300 then
		fightMgr = self.eliteFightMgr
	elseif msg.fightResult.result < 400 then
		fightMgr = self.quarryFightMgr
		local ui = GetLuaUI('DlgQuarry')
		if ui ~= nil then
			DlgQuarryFight:Show(ui:GetRootLayer())
		end
	elseif msg.fightResult.result < 500 then
		fightMgr = self.abyssFightMgr
		-- local ui = GetLuaUI('AbyssChallenge')
		-- if ui ~= nil then
			-- DlgAbyssFight:Show(ui:GetRootLayer())
		-- end
	end
	fightMgr:SetMagData(msg)
	fightMgr:FightBegin()
	while msg.timeLeft > 0 do
		msg.timeLeft = msg.timeLeft - 1
		fightMgr:ShowMsg()
	end
	return false
end

function Player:IsQuarryFighting()
	if #self.quarryFightMsgs > 0 then
		return true
	end
	if self.quarryFightMgr:Valid() then
		return true
	end
	local waitTick = self.quarryFightMgr.waitTicks
	if waitTick and waitTick > 0 then
		return true
	end
	return false
end

function Player:IsAbyssFighting()
	if #self.abyssFightMsgs > 0 then
		return true
	end
	if self.abyssFightMgr:Valid() then
		return true
	end
	local waitTick = self.abyssFightMgr.waitTicks
	if waitTick and waitTick > 0 then
		return true
	end
	return false
end

function Player:regMsgHandler()
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_CharacterData, self, self.onRecvPlayerData)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_SyncPlayerStatus, self, self.onRecvSyncStatus)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_MercenaryAdd, self, self.onRecvMercenaryAdd)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_UpdateMercenaryEquip, self, self.onRecvSyncMercEquip)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_MercenaryAbilityChange, self, self.onRecvMercAbilityChange)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_SetSkill, self, self.onRecvSyncSetSkill)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_RefreshWaitBuildEquip, self, self.onRecvBuildEquip)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_SyncFightData, self, self.onRecvSyncFightData)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_SyncInitFightData, self, self.onRecvInitFightData)
	regJMsgHandler('EquipRemark_Cfg', self, self.onRecvRemakeCfg)
	regJMsgHandler('sign_list', self, self.onRecvSignList)
	regJMsgHandler('guide', self, self.onRecvGuideMsg)
	regJMsgHandler('dl_gift', self, self.onRecvDLGift)
	
	regJMsgHandler('runeskill', self, self.onRecvRuneSkill)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_CharacterDataLuaEx, self, self.onRecvPlayerDataLuaEx)
	regCMsgHandler(proto.S2C_GAME_SERVER.eS2C_SyncPlayerStatusLuaEx, self, self.onRecvSyncStatusLuaEx)
end

function Player:onRecvRuneSkill(data)
	self.runeskill = data
end

function Player:ShowRuneInfo()
	local tlen = 0
	for k, v in pairs(self.runebackpack) do tlen = tlen + 1 end
	tlen = 0
	for k, v in pairs(self.runeinventory) do tlen= tlen + 1 end
	
	print_t(self.runeinventory_pos)
end

function Player:onRecvSyncStatusLuaEx(msg)
	local itemUpdate = {}
	for k, v in ipairs(msg.runeitems) do
		if not v.item then
			self:onRuneItemUpdate(v.where, v.uniqueId, v.item, v.source, itemUpdate)
		end
	end
	for k, v in ipairs(msg.runeitems) do
		if v.item then
			self:onRuneItemUpdate(v.where, v.uniqueId, v.item, v.source, itemUpdate)
		end
	end
	performWithDelay(CCDirector:sharedDirector():getRunningScene(),
		function ()
			if #itemUpdate > 0 then
				for o, f in pairs(self.runeitemSyncHandler) do
					for _, p in ipairs(itemUpdate) do
						f(o, unpack(p))
					end
				end
			end
		end,
		0.001)
		
	--self:ShowRuneInfo()
end

function Player:GetRuneItemSlot(uniqueid)
	for k=1,#self.runeinventory_pos do
		if self.runeinventory_pos[k] == uniqueid then
			return k
		end
	end
	return 0
end

--检测是否已经装备了同ID的符文
function Player:CheckRuneInventoryHave(runeitemid)
	for k, v in pairs(self.runeinventory) do
		if v.templateId == runeitemid then
			return true
		end
	end
	return false
end

function Player:onRecvPlayerDataLuaEx(msg)
	self.runebackpack = {}	
	for k, v in ipairs(msg.playerRunePackage.runebackpack) do
		self.runebackpack[v.uniqueId] = v
	end
	
	self.runeinventory = {}
	for k, v in ipairs(msg.playerRunePackage.runeinventory) do
		self.runeinventory[v.uniqueId] = v
	end
	
	self.runeinventory_pos = msg.playerRunePackageData.runeinventory
	
	--self:ShowRuneInfo()
end

function Player:SortRuneBag()
	self.runebackpackSort = nil
	self.runebackpackSort ={}
	for k, v in pairs(self.runebackpack) do
		self.runebackpackSort[#self.runebackpackSort + 1] = v.uniqueId
	end
	
	table.sort(self.runebackpackSort, function (a, b)
		local aitem = g_Player:GetRuneItem(a)
		local bitem = g_Player:GetRuneItem(b)
		if not aitem then 
			return true
		elseif not bitem then 
			return false 
		end
		if aitem.templateId < bitem.templateId then return true end
		return aitem:Level() > bitem:Level()
	end)
end
function Player:onRecvPlayerData(msg)
	self:ctor()
	self.abilityData = {}
	for k, v in ipairs(msg.playerStatus.abilityData) do
		self.abilityData[k] = v
	end
	self.baseData = {}
	for k, v in pairs(msg.playerFullData.playerBaseData) do
		if type(v) == 'userdata' then
			v = v:tonumber()
		end
		self.baseData[k] = v
	end
	self.backpack = {}
	for k, v in ipairs(msg.playerPackage.backpack) do
		self.backpack[v.uniqueId] = v
	end
	self.inventory = {}
	for k, v in ipairs(msg.playerPackage.inventory) do
		self.inventory[v.uniqueId] = v
	end
	self.warehouse = {}
	for k, v in ipairs(msg.playerPackage.warehouse) do
		self.warehouse[v.uniqueId] = v
	end
	self.mercenaryEuqip = {}
	for k, v in ipairs(msg.playerPackage.mercenaryEuqip) do
		self.mercenaryEuqip[v.uniqueId] = v
	end
	self.mercenaries = {}
	for k, v in ipairs(msg.mercenaryList.mercenaries) do
		self.mercenaries[#self.mercenaries+1] = v
	end
	performWithDelay(CCDirector:sharedDirector():getRunningScene(),
		function ()
			self:UpdateBetterEquip()
			self:UpdateBetterEquipMerc()
		end,
		0.001)
end

function Player:GetRuneInvertorySlotItem(slot)
	--越界
	if slot <=0 or slot > #self.runeinventory_pos then return nil end
	local uid = self.runeinventory_pos[slot]
	local ret = self.runeinventory[uid]
	if ret ~= nil then
		return ret
	end
	return nil
end

function Player:GetRuneItem(uid)
	local ret
	ret = self.runebackpack[uid]
	if ret ~= nil then
		return ret, proto.PACKAGE_TYPE_RUNEBACKPACK
	end
	ret = self.runeinventory[uid]
	if ret ~= nil then
		return ret, proto.PACKAGE_TYPE_RUNEINVENTORY
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE
end

function Player:onRuneItemUpdate(where, uid, item, source, itemUpdate)
	local ret_where = proto.PACKAGE_TYPE_MAX_PACKAGE
	local syncType, prevItem
	if not item then
		prevItem, ret_where = self:RemoveRuneItem(uid,source)
		if prevItem then
			syncType = IUEVENT_REMOVE
		else
			return
		end
	else
		prevItem, ret_where = self:UpdateRuneItem(item)
		if prevItem then
			syncType = IUEVENT_UPDATE
		elseif self:AddRuneItem(item, where, source) then
			ret_where = where
			syncType = IUEVENT_ADD
		else
			return
		end
	end
	itemUpdate[#itemUpdate + 1] = {syncType, ret_where, uid, item, prevItem,source}
end

function Player:onItemUpdate(where, uid, item, source, itemUpdate)
	local ret_where = proto.PACKAGE_TYPE_MAX_PACKAGE
	local syncType, prevItem
	if not item then
		prevItem, ret_where = self:RemoveItem(uid)
		if prevItem then
			syncType = IUEVENT_REMOVE
		else
			return
		end
	else
		prevItem, ret_where = self:UpdateItem(item)
		if prevItem then
			syncType = IUEVENT_UPDATE
		elseif self:AddItem(item, where, source) then
			ret_where = where
			syncType = IUEVENT_ADD
		else
			return
		end
	end
	itemUpdate[#itemUpdate + 1] = {syncType, ret_where, uid, item, prevItem}
end

function Player:onRecvSyncStatus(msg)
	self.syncFlags = msg.flags
	local itemUpdate = {}
	for k, v in ipairs(msg.items) do
		if not v.item then
			self:onItemUpdate(v.where, v.uniqueId, v.item, v.source, itemUpdate)
		end
	end
	for k, v in ipairs(msg.items) do
		if v.item then
			self:onItemUpdate(v.where, v.uniqueId, v.item, v.source, itemUpdate)
		end
	end
	local propUpdate = {}
	for k, v in ipairs(msg.props) do
		self:onPropUpdate(v.type, v.curvalue:tonumber(), propUpdate)
	end
	local abilityData = self.abilityData
	for k, v in ipairs(msg.abilitys) do
		self:onAbilityUpdate(v.type, v.curvalue, abilityData)
	end
	performWithDelay(CCDirector:sharedDirector():getRunningScene(),
		function ()
			if #itemUpdate > 0 then
				for o, f in pairs(self.itemSyncHandler) do
					for _, p in ipairs(itemUpdate) do
						f(o, unpack(p))
					end
				end
				self:UpdateBetterEquip()
				self:UpdateBetterEquipMerc()
			end
			if #propUpdate > 0 then
				for o, f in pairs(self.propSyncHandler) do
					for _, p in ipairs(propUpdate) do
						f(o, unpack(p))
					end
				end
			end
		end,
		0.001)
end

function Player:onRecvMercenaryAdd(msg)
	for k, v in ipairs(msg.mercenaryList.mercenaries) do
		self.mercenaries[#self.mercenaries+1] = v
	end
end

function Player:onRecvSyncMercEquip(msg)
	local id = msg.id
	local index = msg.index+1
	local itemId = msg.itemId
	for k, v in ipairs(self.mercenaries) do
		if v.baseData.id == msg.id then
			v.baseData.inventory[index] = itemId
			break
		end
	end
end

function Player:onRecvMercAbilityChange(msg)
	local showFloat = (g_CurLayer and g_CurLayer.__cname == 'LayerPet')
	for _, merc in ipairs(self.mercenaries) do
		if merc.baseData.id == msg.id then
			local abilityData = merc.statusData.abilityData
			for _, v in ipairs(msg.abilitys) do
				if showFloat then
					self:onAbilityUpdate(v.type, v.curvalue, abilityData)
				else
					abilityData[v.type+1] = v.curvalue
				end
			end
			break
		end
	end
end

function Player:onRecvSyncSetSkill(msg)
	if msg.type == 0 then
		for k, v in ipairs(msg.skillList) do
			self.baseData.skillSet[k] = v
		end
	elseif msg.type == 1 then
		for k, v in ipairs(msg.skillList) do
			self.baseData.skillArena[k] = v
		end
	elseif msg.type == 2 then
		for k, v in ipairs(msg.skillList) do
			self.baseData.skillGroup[k] = v
		end
	elseif msg.type == 3 then
		for k, v in ipairs(msg.skillList) do
			self.baseData.skillBoss[k] = v
		end
	end
end

function Player:onRecvBuildEquip(msg)
	self.buildEquipCost = 500
	local equip = msg.equip
	if not equip then return end
	
	local color = equip:getColorIdx()
	if color == 4 then
		self.buildEquipCost = 1500
	elseif color == 5 then
		self.buildEquipCost = 5000
	end
end


----------------------------------------------------------------
----------------------------------------------------------------
-- 道具相关
function Player:AddItem(item, where, source)
	if where == PACKAGE_TYPE_INVENTORY then
		self.inventory[item.uniqueId] = item
		return true
	end
	if where == PACKAGE_TYPE_BACKPACK then
		self.backpack[item.uniqueId] = item
		if source == ITEM_SOURCE_PICKUP or
			source == ITEM_SOURCE_REMAKE_BUILD or
			source == ITEM_SOURCE_USE_FEEDBACK then
			if self.syncFlags == 0 then
				pushFloatInfo(SSFmt(TDGETSTRCS(item.templateId > 1000 and 'SD_GetEquip' or 'SD_GetItem'), item:getColorName()))
			end
		end
		if source == ITEM_SOURCE_REMAKE then
			pushFloatInfo(SSFmt(TDGETSTRCS('REMAKE_SUCC'), item:getColorName()))
		end
		return true
	end
	if where == PACKAGE_TYPE_WAREHOUSE then
		self.warehouse[item.uniqueId] = item
		if source == ITEM_SOURCE_PICKUP or
			source == ITEM_SOURCE_REMAKE_BUILD or
			source == ITEM_SOURCE_USE_FEEDBACK or
			source == ITEM_SOURCE_ACHIEVEMENT then
			if self.syncFlags == 0 then
				pushFloatInfo(SSFmt(TDGETSTRCS(item.templateId > 1000 and 'SD_GetEquip' or 'SD_GetItem'), item:getColorName()))
			end
		end
		return true
	end
	if where == PACKAGE_TYPE_MERCINVENTORY then
		self.mercenaryEuqip[item.uniqueId] = item
		return true
	end
	return false
end

function Player:GetRuneItemLevel(uiexp)
	local level = 0
	local expr = uiexp
	for k, v in ipairs(self.runeexp) do
		if expr < v then break end
		level = k
		expr = expr - v
	end
	return level, expr
end

function Player:UpdateRuneinventory_pos(slot,uniqueid)
	self.runeinventory_pos[slot] = uniqueid--更新
end

function Player:AddRuneItem(item, where, source)
	if where == proto.PACKAGE_TYPE_RUNEINVENTORY then
		self.runeinventory[item.uniqueId] = item
		self:UpdateRuneinventory_pos(source,item.uniqueId)
		return true
	end
	if where == proto.PACKAGE_TYPE_RUNEBACKPACK then
		self.runebackpack[item.uniqueId] = item
		if source ~= ITEM_SOURCE_NOT_CARE then
			pushFloatInfo(SSFmt(TDGETSTRCS('SD_GetItem'), item:LevelStr()..item:getColorName()))
		end
		self:SortRuneBag()
		return true
	end
	return false
end

function Player:RemoveRuneBackpack(uid)
	local ret = self.runebackpack[uid]
	self.runebackpack[uid] = nil
	return ret
end

function Player:RemoveRuneInventory(uid,source)
	local ret = self.runeinventory[uid]
	self.runeinventory[uid] = nil
	return ret
end

function Player:RemoveRuneItem(uniqueId,source)
	local ret
	-- 符文装备
	ret = self:RemoveRuneInventory(uniqueId)
	if ret~=nil then
		--是装备的
		self:UpdateRuneinventory_pos(source,0)
		return ret, proto.PACKAGE_TYPE_RUNEINVENTORY
	end
	--符文背包
	ret = self:RemoveRuneBackpack(uniqueId)
	if ret~=nil then
		self:SortRuneBag()
		return ret, proto.PACKAGE_TYPE_RUNEBACKPACK
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE

end


function Player:UpdateRuneInventory(item)
	local ret = self.runeinventory[item.uniqueId]
	if ret ~= nil then
		self.runeinventory[item.uniqueId] = item
	end
	return ret
end

function Player:UpdateRuneBackpack(item)
	local ret = self.runebackpack[item.uniqueId]
	if ret ~= nil then
		self.runebackpack[item.uniqueId] = item
	end
	return ret
end

function Player:UpdateRuneItem(item)
	local ret
	--符文装备
	ret = self:UpdateRuneInventory(item)
	if ret ~= nil then
		return ret, proto.PACKAGE_TYPE_RUNEINVENTORY
	end
	--符文背包
	ret = self:UpdateRuneBackpack(item)
	if ret ~= nil then
		return ret, proto.PACKAGE_TYPE_RUNEBACKPACK
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE
end

function Player:RemoveItem(uniqueId)
	local ret
	-- 身上的装备
	ret = self:RemoveInventory(uniqueId)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_INVENTORY
	end
	-- 背包中的装备
	ret = self:RemoveBackpack(uniqueId)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_BACKPACK
	end
	-- 背包中的道具
	ret = self:RemoveWarehouse(uniqueId)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_WAREHOUSE
	end
	-- 佣兵的装备
	ret = self:RemoveMercenary(uniqueId)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_MERCINVENTORY
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE
end

function Player:RemoveInventory(uid)
	local ret = self.inventory[uid]
	self.inventory[uid] = nil
	return ret
end
function Player:RemoveBackpack(uid)
	local ret = self.backpack[uid]
	self.backpack[uid] = nil
	return ret
end
function Player:RemoveWarehouse(uid)
	local ret = self.warehouse[uid]
	self.warehouse[uid] = nil
	return ret
end
function Player:RemoveMercenary(uid)
	local ret = self.mercenaryEuqip[uid]
	self.mercenaryEuqip[uid] = nil
	return ret
end


function Player:UpdateItem(item)
	local ret
	-- 身上的装备
	ret = self:UpdateInventory(item)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_INVENTORY
	end
	-- 背包中的装备
	ret = self:UpdateBackpack(item)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_BACKPACK
	end
	-- 背包中的道具
	ret = self:UpdateWarehouse(item)
	if ret ~= nil then
		if item.templateId < 1000 and self.syncFlags == 0 then
			if item.stackCount > ret.stackCount then
				pushFloatInfo(SSFmt(TDGETSTRCS('SD_GetItem'),
					SSFmt('%s*%u', item:GetConfig().name, item.stackCount - ret.stackCount)
				))
			end
		end
		return ret, PACKAGE_TYPE_WAREHOUSE
	end
	-- 佣兵的装备
	ret = self:UpdateMercenary(item)
	if ret ~= nil then
		return ret, PACKAGE_TYPE_MERCINVENTORY
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE
end

function Player:UpdateInventory(item)
	local ret = self.inventory[item.uniqueId]
	if ret ~= nil then
		self.inventory[item.uniqueId] = item
	end
	return ret
end
function Player:UpdateBackpack(item)
	local ret = self.backpack[item.uniqueId]
	if ret ~= nil then
		self.backpack[item.uniqueId] = item
	end
	return ret
end
function Player:UpdateWarehouse(item)
	local ret = self.warehouse[item.uniqueId]
	if ret ~= nil then
		self.warehouse[item.uniqueId] = item
	end
	return ret
end
function Player:UpdateMercenary(item)
	local ret = self.mercenaryEuqip[item.uniqueId]
	if ret ~= nil then
		self.mercenaryEuqip[item.uniqueId] = item
	end
	return ret
end

function Player:checkBetterEquip(item, equips)
	if not item then return end
	local cfg = item:GetConfig()
	if not cfg then return end
	if cfg.itemClass ~= ITEM_CLASS_EQUIPMENT then
		return
	end
	if cfg.requireRace ~= RACE_TYPE_MAX and
		cfg.requireRace ~= self.baseData.race then
			return
	end
	if cfg.level > self.baseData.level + 5 then
		return
	end
	local equipType = cfg.equipType
	local item2 = equips[equipType]
	if not item2 or item:GetScore() > item2:GetScore() then
		equips[equipType] = item
	end
end
function Player:UpdateBetterEquip()
	local temp = {}
	for k, v in pairs(self.inventory) do
		temp[v:GetConfig().equipType] = v
	end
	local temp2 = table.copy(temp)
	for k, v in pairs(self.backpack) do
		self:checkBetterEquip(v, temp)
	end
	local betterEquip, hasBetter = {}, false
	for k, v in pairs(temp) do
		if v ~= temp2[k] then
			betterEquip[k] = v.uniqueId
			hasBetter = true
		end
	end
	self.betterEquip = betterEquip
	local layerRoot = LayerRoot:sharedLayerRoot()
	if layerRoot then
		layerRoot:updateBetterEquipState(hasBetter)
	end
end
local mercEquipType = {
	[EQUIPMENT_TYPE_MAIN_WEAPON] = true,
	[EQUIPMENT_TYPE_SUB_WEAPON] = true,
	[EQUIPMENT_TYPE_BODY] = true,
	[EQUIPMENT_TYPE_LEG] = true,
}
function Player:checkBetterEquipMerc(merc, item, equips)
	if not item then return end
	local cfg = item:GetConfig()
	if not cfg then return end
	if cfg.itemClass ~= ITEM_CLASS_EQUIPMENT then
		return
	end
	if not mercEquipType[cfg.equipType] then
		return
	end
	if cfg.requireRace ~= RACE_TYPE_MAX and
		cfg.requireRace ~= mercCfg[merc.baseData.id].race then
			return
	end
	if cfg.level > self.baseData.level + 5 then
		return
	end
	local equipType = cfg.equipType
	local item2 = equips[equipType]
	if not item2 or item:GetScore() > item2:GetScore() then
		equips[equipType] = item
	end
end
function Player:UpdateBetterEquipMerc()
	local betterEquipMerc, hasBetter = {}, false
	for idx, merc in ipairs(self.mercenaries) do
		local temp = {}
		for _, uid in ipairs(merc.baseData.inventory) do
			local equip = self.mercenaryEuqip[uid]
			if equip then
				temp[equip:GetConfig().equipType] = equip
			end
		end
		local temp2 = table.copy(temp)
		for k, v in pairs(self.backpack) do
			self:checkBetterEquipMerc(merc, v, temp)
		end
		betterEquipMerc[idx] = {}
		for k, v in pairs(temp) do
			if v ~= temp2[k] then
				betterEquipMerc[idx][k] = v.uniqueId
				hasBetter = true
			end
		end
	end
	self.betterEquipMerc = betterEquipMerc
	local layerRoot = LayerRoot:sharedLayerRoot()
	if layerRoot then
		layerRoot:updateBetterEquipStateMerc(hasBetter)
	end
end
function Player:HasBetterEquip()
	for k, v in pairs(self.betterEquip) do
		return true
	end
	return false
end
function Player:HasBetterEquipMerc(id)
	for idx, equips in pairs(self.betterEquipMerc) do
		if id == nil or self.mercenaries[idx].baseData.id == id then
			for _ in pairs(equips) do
				return true
			end
		end
	end
	return false
end
function Player:ChangeAllBetterEquip()
	local msg = proto.C2S_PutOnEquipment.new()
	for k, v in pairs(self.betterEquip) do
		msg.uniqueId = v
		msg.isForce = true
		msg.isRight = false
		g_SendMsg(msg)
	end
end
function Player:ChangeAllBetterEquipMerc(idx)
	if not self.mercenaries[idx] then return end
	local msg = proto.C2S_MercenaryPutOnEquip.new()
	msg.id = self.mercenaries[idx].baseData.id
	for k, v in pairs(self.betterEquipMerc[idx]) do
		msg.itemId = v
		g_SendMsg(msg)
	end
end

function Player:GetItemCount(templateId)
	local ret = 0
	for k, v in pairs(self.warehouse) do
		if v.templateId == templateId then
			ret = ret + v.stackCount
		end
	end
	return ret
end

function Player:GetItem(uid)
	local ret
	ret = self.backpack[uid]
	if ret ~= nil then
		return ret, PACKAGE_TYPE_BACKPACK
	end
	ret = self.warehouse[uid]
	if ret ~= nil then
		return ret, PACKAGE_TYPE_WAREHOUSE
	end
	ret = self.inventory[uid]
	if ret ~= nil then
		return ret, PACKAGE_TYPE_INVENTORY
	end
	ret = self.mercenaryEuqip[uid]
	if ret ~= nil then
		return ret, PACKAGE_TYPE_MERCINVENTORY
	end
	return nil, proto.PACKAGE_TYPE_MAX_PACKAGE
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 基本属性相关
local propName = {
	'level',
	'exp',
	'sceneId',
	'cristal',
	'money',
	'mapLine',
	'lastOlDay',
	'skillSet',
	'packageCapacity',
	'remakeValue',
	'gemExp',
	'battleSet',
	'totalCharge',
	'quickFightTimes',
	'activeMercenary',
	'arenaChallengeTimes',
	'honor',
	'remakeRefreshTimes',
	'mallBuyCoinTimes',
	'bossChallengeTimes',
	'mallRefreshTimes',
	'mCardLimitDay',
	'mCardGet',
}
local tipInfo = {
	{'SYNC_PROP_TYPE_LEVEL', 2},
	{'SYNC_PROP_TYPE_EXP',},
	{'SYNC_PROP_TYPE_SCENEID',},
	{'SYNC_PROP_TYPE_CRISTAL', 1},
	{'SYNC_PROP_TYPE_MONEY', 1},
	{'SYNC_PROP_TYPE_MAPLINE',},
	{'SYNC_PROP_TYPE_LASTOLDAY',},
	{'SYNC_PROP_TYPE_SKILLSET',},
	{'SYNC_PROP_TYPE_PACKAGECAPACITY', 3},
	{'SYNC_PROP_TYPE_REMAKEVALUE', 1},
	{'SYNC_PROP_TYPE_GEMEXP',},
	{'SYNC_PROP_TYPE_BATTLESET',},
	{'SYNC_PROP_TYPE_VIPLEVEL',},
	{'SYNC_PROP_TYPE_QUICKFIGHTTIMES',},
	{'SYNC_PROP_TYPE_ACTIVEMERCENARY',},
	{'SYNC_PROP_TYPE_ARENACHALLENGETIMES',},
	{'SYNC_PROP_TYPE_HONOR', 1},
	{'SYNC_PROP_TYPE_remakeRefreshTimes',},
	{'SYNC_PROP_TYPE_mallBuyCoinTimes',},
	{'SYNC_PROP_TYPE_bossChallengeTimes', 1},
}
function Player:onPropUpdate(tp, cur, propUpdate)
	if tp < 0 or tp >= #propName then return end
	
	local pre = self.baseData[propName[tp+1]]
	if not pre or pre == cur then return end
	self.baseData[propName[tp+1]] = cur
	propUpdate[#propUpdate+1] = {tp, pre, cur}
	
	if self.syncFlags == 1 and tp ~= SYNC_PROP_TYPE_honor then
		return
	end
	
	local info = tipInfo[tp+1]
	if not info then return end
	
	if info[2] == 1 then
		if pre < cur then
			pushFloatInfo(SSFmt('%s <c=00ff00>+ %u', TDGETSTRCS(info[1]), cur - pre))
		else
			pushFloatInfo(SSFmt('%s <c=00ff00>- %u', TDGETSTRCS(info[1]), pre - cur))
		end
	elseif info[2] == 2 then
		pushFloatInfo(SSFmt(TDGETSTRCS(info[1]), cur))
	elseif info[2] == 3 then
		pushFloatInfo(SSFmt(TDGETSTRCS(info[1]), cur - pre))
	end
end
local abilityName = {
	'ABILITY_TYPE_POWER',
	'ABILITY_TYPE_AGILITY',
	'ABILITY_TYPE_INTELLIGENCE',
	'ABILITY_TYPE_ENDURANCE',
	'ABILITY_TYPE_ARMOR',
	'ABILITY_TYPE_HP_MAX',
	'ABILITY_TYPE_MP_MAX',
	'ABILITY_TYPE_DAMAGE_MIN',
	'ABILITY_TYPE_DAMAGE_MAX',
	'ABILITY_TYPE_PHYSICS_DEF',
	'ABILITY_TYPE_MAGIC_DEF',
	'ABILITY_TYPE_CRITICAL',
	'ABILITY_TYPE_TENACITY',
	'ABILITY_TYPE_ACCURACY',
	'ABILITY_TYPE_DODGE',
	'ABILITY_TYPE_MP_RECOVER',
	'ABILITY_TYPE_BATTLE_POWER',
	'ABILITY_TYPE_S_HPRECOVER',
	'ABILITY_TYPE_S_DROPEQUIP',
	'ABILITY_TYPE_S_GLOD',
	'ABILITY_TYPE_S_CRITICAL',
	'ABILITY_TYPE_S_HP_MAX',
	'ABILITY_TYPE_S_EXP',
	'ABILITY_TYPE_S_RESIST_DB',
	'ABILITY_TYPE_S_DRINK_HP',
	'ABILITY_TYPE_S_SKIP_ARMOR',
	'ABILITY_TYPE_S_DAMAGE_APP',
}

function Player:getAbilityStr(tp,value,token)
	if tp < ABILITY_TYPE_S_HPRECOVER then
		return SSFmt('<c=00ff00>%s %s%u', TDGETSTRCS(abilityName[tp+1]),token, value)
	end
	return SSFmt('<c=00ff00>%s %s%.02f％', TDGETSTRCS(abilityName[tp+1]), token, value/100);
end

function Player:getSq2Str(tp,value,token)
	return SSFmt('<c=00ff00>%s %s%.02f%%', TDGETSTRCS(SSFmt('SYNC_PROP_SQ2_%d',tp)),token, value/100)
end

function Player:onAbilityUpdate(tp, cur, abilityData)

--print("Player:onAbilityUpdate1")

	tp = tp + 1
	if tp < 1 or tp > ABILITY_TYPE_ABILITY_COUNT then return end
	
	local pre = abilityData[tp]
	if pre == cur then return end
	abilityData[tp] = cur
	if self.syncFlags ~= 0 then return end
	if tp-1 == ABILITY_TYPE_BATTLE_POWER then return end
	
	
	--print("Player:onAbilityUpdate2")
	
	local info
	if tp <= ABILITY_TYPE_S_HPRECOVER then
		if pre < cur then
			info = SSFmt('<c=00ff00>%s + %u', TDGETSTRCS(abilityName[tp]), cur - pre);
		else
			info = SSFmt('<c=ff0000>%s - %u', TDGETSTRCS(abilityName[tp]), pre - cur);
		end
	else
		if pre < cur then
			info = SSFmt('<c=00ff00>%s + %.02f%%', TDGETSTRCS(abilityName[tp]), (cur - pre)/100);
		else
			info = SSFmt('<c=ff0000>%s - %.02f%%', TDGETSTRCS(abilityName[tp]), (pre - cur)/100);
		end
	end
	
	--print("Player:onAbilityUpdate3")
	pushFloatInfo(info)
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 装备配备
function Player:onRecvRemakeCfg(msg)
	self.sqeat = msg.SqEat
	self.sqinherit = msg.SqInherit
	self.sqexp = msg.SqExp
	self.sq2eat = msg.Sq2Eat
	self.sq2inherit = msg.Sq2Inherit
	self.sq2exp = msg.Sq2Exp
	self.stareat = msg.StarEat
	self.starinherit = msg.StarInherit
	self.runeexp = msg.RuneExp
	
	--收到消息了排序哈
	self:SortRuneBag()
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 每日签到
function Player:onRecvSignList(msg)
	self.signList = msg.list
	self.signTick = msg.tick
end
function Player:GetSignList()
	g_SendMsg({ptype = 'sign_list', tick = self.signTick or 0})
end

----------------------------------------------------------------
----------------------------------------------------------------
-- 新手引导
function Player:onRecvGuideMsg(msg)
	local cfg = guideCfg[msg]
	if cfg then
		guideMgr.ShowGuide(cfg[1])
	end
	return false
end

function Player:onRecvDLGift(show)
	self.hasDLGift = show
end

g_Player = g_Player or Player.new()
g_Player:regMsgHandler()